(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/hack-session.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var Crypto = Npm.require("crypto");                                    // 17
var Http = Npm.require("http");                                        // 18
var Https = Npm.require("https");                                      // 19
var Future = Npm.require("fibers/future");                             // 20
var Net = Npm.require("net");                                          // 21
var Dgram = Npm.require("dgram");                                      // 22
var Promise = Npm.require("es6-promise").Promise;                      // 23
var Capnp = Npm.require("capnp");                                      // 24
                                                                       //
var EmailRpc = Capnp.importSystem("sandstorm/email.capnp");            // 26
var HackSessionContext = Capnp.importSystem("sandstorm/hack-session.capnp").HackSessionContext;
var Supervisor = Capnp.importSystem("sandstorm/supervisor.capnp").Supervisor;
var SystemPersistent = Capnp.importSystem("sandstorm/supervisor.capnp").SystemPersistent;
var IpRpc = Capnp.importSystem("sandstorm/ip.capnp");                  // 30
var EmailSendPort = EmailRpc.EmailSendPort;                            // 31
var Grain = Capnp.importSystem("sandstorm/grain.capnp");               // 32
                                                                       //
var Url = Npm.require("url");                                          // 34
                                                                       //
ROOT_URL = Url.parse(process.env.ROOT_URL);                            // 36
HOSTNAME = ROOT_URL.hostname;                                          // 37
                                                                       //
SessionContextImpl = (function () {                                    // 39
  function SessionContextImpl(grainId, sessionId, identityId) {        // 40
    babelHelpers.classCallCheck(this, SessionContextImpl);             //
                                                                       //
    this.grainId = grainId;                                            // 41
    this.sessionId = sessionId;                                        // 42
    this.identityId = identityId;                                      // 43
  }                                                                    //
                                                                       //
  SessionContextImpl.prototype.offer = (function () {                  // 39
    function offer(cap, requiredPermissions, descriptor, displayInfo) {
      var _this2 = this;                                               //
                                                                       //
      return inMeteor(function () {                                    // 47
        if (!_this2.identityId) {                                      // 48
          // TODO(soon): allow non logged in users?                    //
          throw new Meteor.Error(400, "Only logged in users can offer capabilities.");
        }                                                              //
                                                                       //
        var castedCap = cap.castAs(SystemPersistent);                  // 53
        var apiTokenOwner = { webkey: null };                          // 54
        var isUiView = descriptor && descriptor.tags && descriptor.tags.length === 1 && descriptor.tags[0] && descriptor.tags[0].id && descriptor.tags[0].id === Grain.UiView.typeId;
        if (isUiView) {                                                // 58
          apiTokenOwner = {                                            // 59
            user: {                                                    // 60
              identityId: _this2.identityId                            // 61
            }                                                          //
          };                                                           //
        }                                                              //
                                                                       //
        // TODO(soon): This will eventually use SystemPersistent.addRequirements when membranes
        // are fully implemented for supervisors.                      //
        // The following fields will be overwritten by PersistentUiView.save(), so no need to
        // pass them in:                                               //
        //lastUsed: new Date(),                                        //
        //title: "", // This will be replaced by the token's title     //
        //denormalizedGrainMetadata: {}, // This will look up the package for the grain referenced.
        var requirement = {                                            // 73
          permissionsHeld: {                                           // 74
            grainId: _this2.grainId,                                   // 75
            identityId: _this2.identityId,                             // 76
            permissions: requiredPermissions                           // 77
          }                                                            //
        };                                                             //
                                                                       //
        if (!checkRequirements([requirement])) {                       // 81
          throw new Meteor.Error(403, "Permissions not satisfied.");   // 82
        }                                                              //
                                                                       //
        var save = castedCap.save(apiTokenOwner);                      // 85
        var sturdyRef = waitPromise(save).sturdyRef;                   // 86
        ApiTokens.update({ _id: hashSturdyRef(sturdyRef) }, { $push: { requirements: requirement } });
        var powerboxView = isUiView ? {                                // 88
          offer: {                                                     // 89
            uiView: {                                                  // 90
              token: sturdyRef.toString(),                             // 91
              tokenId: hashSturdyRef(sturdyRef.toString())             // 92
            }                                                          //
          }                                                            //
        } : {                                                          //
          offer: {                                                     // 96
            url: ROOT_URL.protocol + "//" + makeWildcardHost("api") + "#" + sturdyRef
          }                                                            //
        };                                                             //
        Sessions.update({ _id: _this2.sessionId }, {                   // 100
          $set: {                                                      // 102
            powerboxView: powerboxView                                 // 103
          }                                                            //
        });                                                            //
      });                                                              //
    }                                                                  //
                                                                       //
    return offer;                                                      //
  })();                                                                //
                                                                       //
  return SessionContextImpl;                                           //
})();                                                                  //
                                                                       //
Meteor.methods({                                                       // 111
  finishPowerboxRequest: function (webkeyUrl, saveLabel, identityId, grainId) {
    check(webkeyUrl, String);                                          // 113
    check(saveLabel, Match.OneOf(undefined, null, String));            // 114
    check(identityId, String);                                         // 115
    check(grainId, String);                                            // 116
                                                                       //
    var userId = Meteor.userId();                                      // 118
    if (!userId || !globalDb.userHasIdentity(userId, identityId)) {    // 119
      throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
    }                                                                  //
                                                                       //
    var parsedWebkey = Url.parse(webkeyUrl.trim());                    // 123
    if (parsedWebkey.host !== makeWildcardHost("api")) {               // 124
      console.log(parsedWebkey.hostname, makeWildcardHost("api"));     // 125
      throw new Meteor.Error(500, "Hostname does not match this server. External webkeys are not " + "supported (yet)");
    }                                                                  //
                                                                       //
    var token = parsedWebkey.hash;                                     // 130
    if (!token) {                                                      // 131
      throw new Meteor.Error(400, "Invalid webkey. You must pass in the full webkey, " + "including domain name and hash fragment");
    }                                                                  //
                                                                       //
    token = token.slice(1);                                            // 136
    var cap = restoreInternal(hashSturdyRef(token), Match.Optional({ webkey: Match.Optional(Match.Any) }), [], new Buffer(token)).cap;
    var castedCap = cap.castAs(SystemPersistent);                      // 140
    var grainOwner = {                                                 // 141
      grainId: grainId,                                                // 142
      introducerIdentity: identityId                                   // 143
    };                                                                 //
    if (saveLabel) {                                                   // 145
      grainOwner.saveLabel = {                                         // 146
        defaultText: saveLabel                                         // 147
      };                                                               //
    }                                                                  //
                                                                       //
    var save = castedCap.save({ grain: grainOwner });                  // 151
    var sturdyRef = waitPromise(save).sturdyRef;                       // 152
                                                                       //
    return sturdyRef.toString();                                       // 154
  },                                                                   //
                                                                       //
  finishPowerboxOffer: function (sessionId) {                          // 157
    check(sessionId, String);                                          // 158
                                                                       //
    Sessions.update({ _id: sessionId }, { $unset: { powerboxView: null } });
  },                                                                   //
                                                                       //
  getViewInfoForApiToken: function (apiTokenId) {                      // 163
    check(apiTokenId, String);                                         // 164
    var token = globalDb.collections.apiTokens.findOne(apiTokenId);    // 165
    if (!token) {                                                      // 166
      throw new Meteor.Error(400, "No such token.");                   // 167
    }                                                                  //
                                                                       //
    var grain = globalDb.collections.grains.findOne(token.grainId);    // 170
    if (!grain) {                                                      // 171
      throw new Meteor.Error(500, "No grain found for token.");        // 172
    }                                                                  //
                                                                       //
    return grain.cachedViewInfo;                                       // 175
  }                                                                    //
});                                                                    //
                                                                       //
HackSessionContextImpl = (function (_SessionContextImpl) {             // 179
  babelHelpers.inherits(HackSessionContextImpl, _SessionContextImpl);  //
                                                                       //
  function HackSessionContextImpl(grainId, sessionId, identityId) {    // 180
    babelHelpers.classCallCheck(this, HackSessionContextImpl);         //
                                                                       //
    _SessionContextImpl.call(this, grainId, sessionId, identityId);    // 181
  }                                                                    //
                                                                       //
  HackSessionContextImpl.prototype._getPublicId = (function () {       // 179
    function _getPublicId() {                                          // 184
      // Get the grain's public ID, assigning a new one if it doesn't yet have one.
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      while (!this.publicId) {                                         // 189
        // We haven't looked up the public ID yet.                     //
        var grain = Grains.findOne(this.grainId, { fields: { publicId: 1 } });
        if (!grain) throw new Error("Grain does not exist.");          // 192
                                                                       //
        if (grain.publicId) {                                          // 194
          this.publicId = grain.publicId;                              // 195
        } else {                                                       //
          // The grain doesn't have a public ID yet. Generate one.     //
          var candidate = generateRandomHostname(20);                  // 198
                                                                       //
          if (Grains.findOne({ publicId: candidate })) {               // 200
            // This should never ever happen.                          //
            console.error("CRITICAL PROBLEM: Public ID collision. " + "CSPRNG is bad or has insufficient entropy.");
            continue;                                                  // 204
          }                                                            //
                                                                       //
          // Carefully perform an update that becomes a no-op if anyone else has assigned a public ID
          // simultaneously.                                           //
          if (Grains.update({ _id: this.grainId, publicId: { $exists: false } }, { $set: { publicId: candidate } }) > 0) {
            // We won the race.                                        //
            this.publicId = candidate;                                 // 212
          }                                                            //
        }                                                              //
      }                                                                //
                                                                       //
      return this.publicId;                                            // 217
    }                                                                  //
                                                                       //
    return _getPublicId;                                               //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype._getAddress = (function () {        // 179
    function _getAddress() {                                           // 220
      // Get the grain's outgoing e-mail address.                      //
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      return this._getPublicId() + "@" + HOSTNAME;                     // 225
    }                                                                  //
                                                                       //
    return _getAddress;                                                //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype._getUserAddress = (function () {    // 179
    function _getUserAddress() {                                       // 228
      // Get the user's e-mail address.                                //
      //                                                               //
      // Must be called in a Meteor context.                           //
                                                                       //
      var grain = Grains.findOne(this.grainId, { fields: { identityId: 1 } });
                                                                       //
      var identity = globalDb.getIdentity(grain.identityId);           // 235
                                                                       //
      var emails = SandstormDb.getVerifiedEmails(identity);            // 237
      var email = emails.length > 0 && emails[0] || identity.unverifiedEmail;
                                                                       //
      var result = {};                                                 // 240
      if (email) {                                                     // 241
        result.address = email;                                        // 242
      }                                                                //
                                                                       //
      if (identity.profile.name) {                                     // 245
        result.name = identity.profile.name;                           // 246
      }                                                                //
                                                                       //
      return result;                                                   // 249
    }                                                                  //
                                                                       //
    return _getUserAddress;                                            //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.send = (function () {               // 179
    function send(email) {                                             // 252
      return hackSendEmail(this, email);                               // 253
    }                                                                  //
                                                                       //
    return send;                                                       //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getPublicId = (function () {        // 179
    function getPublicId() {                                           // 256
      return inMeteor((function () {                                   // 257
        var result = {};                                               // 258
                                                                       //
        result.publicId = this._getPublicId();                         // 260
        result.hostname = HOSTNAME;                                    // 261
        result.autoUrl = ROOT_URL.protocol + "//" + makeWildcardHost(result.publicId);
                                                                       //
        var grain = Grains.findOne(this.grainId, { fields: { userId: 1 } });
        result.isDemoUser = Meteor.users.findOne(grain.userId).expires ? true : false;
                                                                       //
        return result;                                                 // 267
      }).bind(this));                                                  //
    }                                                                  //
                                                                       //
    return getPublicId;                                                //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.httpGet = (function () {            // 179
    function httpGet(url) {                                            // 271
      var _this = this;                                                // 272
      var session = _this;                                             // 273
                                                                       //
      return new Promise(function (resolve, reject) {                  // 275
        var requestMethod = Http.request;                              // 276
        if (url.indexOf("https://") === 0) {                           // 277
          requestMethod = Https.request;                               // 278
        } else if (url.indexOf("http://") !== 0) {                     //
          err = new Error("Protocol not recognized.");                 // 280
          err.nature = "precondition";                                 // 281
          reject(err);                                                 // 282
        }                                                              //
                                                                       //
        req = requestMethod(url, function (resp) {                     // 285
          var buffers = [];                                            // 286
          var err = undefined;                                         // 287
                                                                       //
          switch (Math.floor(resp.statusCode / 100)) {                 // 289
            case 2:                                                    // 290
              // 2xx response -- OK.                                   //
              resp.on("data", function (buf) {                         // 291
                buffers.push(buf);                                     // 292
              });                                                      //
                                                                       //
              resp.on("end", function () {                             // 295
                resolve({                                              // 296
                  content: Buffer.concat(buffers),                     // 297
                  mimeType: resp.headers["content-type"] || null       // 298
                });                                                    //
              });                                                      //
              break;                                                   // 301
            case 3:                                                    // 301
              // 3xx response -- redirect.                             //
              resolve(session.httpGet(resp.headers.location));         // 303
              break;                                                   // 304
            case 4:                                                    // 304
              // 4xx response -- client error.                         //
              err = new Error("Status code " + resp.statusCode + " received in response.");
              err.nature = "precondition";                             // 307
              reject(err);                                             // 308
              break;                                                   // 309
            case 5:                                                    // 310
              // 5xx response -- internal server error.                //
              err = new Error("Status code " + resp.statusCode + " received in response.");
              err.nature = "localBug";                                 // 312
              reject(err);                                             // 313
              break;                                                   // 314
            default:                                                   // 314
              // ???                                                   //
              err = new Error("Invalid status code " + resp.statusCode + " received in response.");
              err.nature = "localBug";                                 // 317
              reject(err);                                             // 318
              break;                                                   // 319
          }                                                            // 319
        });                                                            //
                                                                       //
        req.on("error", function (e) {                                 // 323
          e.nature = "networkFailure";                                 // 324
          reject(e);                                                   // 325
        });                                                            //
                                                                       //
        req.setTimeout(15000, function () {                            // 328
          req.abort();                                                 // 329
          err = new Error("Request timed out.");                       // 330
          err.nature = "localBug";                                     // 331
          err.durability = "overloaded";                               // 332
          reject(err);                                                 // 333
        });                                                            //
                                                                       //
        req.end();                                                     // 336
      });                                                              //
    }                                                                  //
                                                                       //
    return httpGet;                                                    //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getUserAddress = (function () {     // 179
    function getUserAddress() {                                        // 340
      return inMeteor((function () {                                   // 341
        return this._getUserAddress();                                 // 342
      }).bind(this));                                                  //
    }                                                                  //
                                                                       //
    return getUserAddress;                                             //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteGenerateApiToken = (function () {
    function obsoleteGenerateApiToken(petname, userInfo, expires) {    // 346
      throw new Error("generateApiToken() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteGenerateApiToken;                                   //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteListApiTokens = (function () {
    function obsoleteListApiTokens() {                                 // 350
      throw new Error("listApiTokens() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteListApiTokens;                                      //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.obsoleteRevokeApiToken = (function () {
    function obsoleteRevokeApiToken(tokenId) {                         // 354
      throw new Error("revokeApiToken() has been removed. Use offer templates instead.");
    }                                                                  //
                                                                       //
    return obsoleteRevokeApiToken;                                     //
  })();                                                                //
                                                                       //
  HackSessionContextImpl.prototype.getUiViewForEndpoint = (function () {
    function getUiViewForEndpoint(url) {                               // 358
      var parsedUrl = Url.parse(url);                                  // 359
                                                                       //
      if (parsedUrl.hash) {                                            // 361
        // Assume that anything with a fragment is a webkey            //
        if (parsedUrl.pathname) {                                      // 362
          throw new Error("Webkey urls cannot contain a path.");       // 363
        }                                                              //
                                                                       //
        var apiHost = ROOT_URL.protocol + "//" + makeWildcardHost("api");
        var urlProtoAndHost = parsedUrl.protocol + "//" + parsedUrl.host;
        var token = parsedUrl.hash.slice(1); // Get rid of # which is always the first character
        if (urlProtoAndHost === apiHost) {                             // 369
          return getWrappedUiViewForToken(token);                      // 370
        } else {                                                       //
          return { view: new ExternalUiView(url, this.grainId, token) };
        }                                                              //
      } else {                                                         //
        return { view: new ExternalUiView(url, this.grainId) };        // 375
      }                                                                //
    }                                                                  //
                                                                       //
    return getUiViewForEndpoint;                                       //
  })();                                                                //
                                                                       //
  return HackSessionContextImpl;                                       //
})(SessionContextImpl);                                                //
                                                                       //
makeHackSessionContext = function (grainId, sessionId, identityId) {   // 380
  return new Capnp.Capability(new HackSessionContextImpl(grainId, sessionId, identityId), HackSessionContext);
};                                                                     //
                                                                       //
var HOSTNAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789";           // 385
                                                                       //
generateRandomHostname = function (length) {                           // 387
  // Generate a random unique name suitable for use in a hostname.     //
                                                                       //
  var digits = [];                                                     // 390
  for (var i = 0; i < length; i++) {                                   // 391
    digits[i] = Random.choice(HOSTNAME_CHARS);                         // 392
  }                                                                    //
                                                                       //
  return digits.join("");                                              // 395
};                                                                     //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=hack-session.js.map
