(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/admin.js                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var ADMIN_TOKEN_EXPIRATION_TIME = 15 * 60 * 1000;                      // 17
var publicAdminSettings = ["google", "github", "emailToken", "splashUrl", "signupDialog", "adminAlert", "adminAlertTime", "adminAlertUrl", "termsUrl", "privacyUrl", "appMarketUrl", "appIndexUrl", "appUpdatesEnabled", "serverTitle", "returnAddress"];
                                                                       //
DEFAULT_SIGNUP_DIALOG = "You've been invited to join this Sandstorm server!";
                                                                       //
var adminRoute = RouteController.extend({                              // 27
  template: "admin",                                                   // 28
  waitOn: function () {                                                // 29
    var subs = [Meteor.subscribe("admin", this.params._token), Meteor.subscribe("adminServiceConfiguration", this.params._token)];
    if (this.params._token) {                                          // 34
      subs.push(Meteor.subscribe("adminToken", this.params._token));   // 35
    }                                                                  //
                                                                       //
    return subs;                                                       // 38
  },                                                                   //
                                                                       //
  data: function () {                                                  // 41
    var adminToken = AdminToken.findOne();                             // 42
    return {                                                           // 43
      settings: Settings.find(),                                       // 44
      token: this.params._token,                                       // 45
      isUserPermitted: isAdmin() || adminToken && adminToken.tokenIsValid
    };                                                                 //
  },                                                                   //
                                                                       //
  action: function () {                                                // 50
    // Test the WILDCARD_HOST for sanity.                              //
    Tracker.nonreactive(function () {                                  // 52
      if (Session.get("alreadyTestedWildcardHost")) {                  // 53
        return;                                                        // 54
      }                                                                //
                                                                       //
      HTTP.call("GET", "//" + makeWildcardHost("selftest-" + Random.hexString(20)), { timeout: 2000 }, function (error, response) {
        Session.set("alreadyTestedWildcardHost", true);                // 59
        var looksGood = undefined;                                     // 60
        if (error) {                                                   // 61
          looksGood = false;                                           // 62
        } else {                                                       //
          if (response.statusCode === 200) {                           // 64
            looksGood = true;                                          // 65
          } else {                                                     //
            console.log("Surpring status code from self test domain", response.statusCode);
            looksGood = false;                                         // 68
          }                                                            //
        }                                                              //
                                                                       //
        Session.set("wildcardHostWorks", looksGood);                   // 72
      });                                                              //
    });                                                                //
                                                                       //
    var state = this.state;                                            // 76
    Meteor.call("getSmtpUrl", this.params._token, function (error, result) {
      state.set("smtpUrl", result);                                    // 78
    });                                                                //
                                                                       //
    var user = Meteor.user();                                          // 81
    if (user && user.loginIdentities) {                                // 82
      if (this.params._token) {                                        // 83
        if (!user.signupKey || !user.isAdmin) {                        // 84
          Meteor.call("signUpAsAdmin", this.params._token);            // 85
        } else if (user.isAdmin) {                                     //
          // We don't need the token. Redirect to the current route, minus the token parameter.
          Router.go(this.route.getName(), {}, _.pick(this.params, "query", "hash"));
        }                                                              //
      }                                                                //
    }                                                                  //
                                                                       //
    resetResult(state);                                                // 93
    state.set("configurationServiceName", null);                       // 94
    state.set("token", this.params._token);                            // 95
    this.render();                                                     // 96
  }                                                                    //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 100
  this.route("adminSettings", {                                        // 101
    path: "/admin/settings/:_token?",                                  // 102
    controller: adminRoute                                             // 103
  });                                                                  //
  this.route("adminUsers", {                                           // 105
    path: "/admin/users/:_token?",                                     // 106
    controller: adminRoute                                             // 107
  });                                                                  //
  this.route("adminStats", {                                           // 109
    path: "/admin/stats/:_token?",                                     // 110
    controller: adminRoute                                             // 111
  });                                                                  //
  this.route("adminLog", {                                             // 113
    path: "/admin/log/:_token?",                                       // 114
    controller: adminRoute                                             // 115
  });                                                                  //
  this.route("adminInvites", {                                         // 117
    path: "/admin/invites/:_token?",                                   // 118
    controller: adminRoute                                             // 119
  });                                                                  //
  this.route("adminCaps", {                                            // 121
    path: "/admin/capabilities/:_token?",                              // 122
    controller: adminRoute                                             // 123
  });                                                                  //
  this.route("adminAdvanced", {                                        // 125
    path: "/admin/advanced/:_token?",                                  // 126
    controller: adminRoute                                             // 127
  });                                                                  //
  this.route("adminOld", {                                             // 129
    path: "/admin/:_token?",                                           // 130
    action: function () {                                              // 131
      this.redirect("adminSettings", this.params);                     // 132
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
var resetResult = function (state) {                                   // 137
  state = state || Iron.controller().state;                            // 138
  state.set("numSettings", 1);                                         // 139
  state.set("successes", 0);                                           // 140
  state.set("failures", 0);                                            // 141
  state.set("errors", []);                                             // 142
  state.set("fadeAlert", false);                                       // 143
  state.set("successMessage", "Your settings have been saved.");       // 144
  state.set("powerboxOfferUrl", null);                                 // 145
};                                                                     //
                                                                       //
var getToken = function () {                                           // 148
  var state = Iron.controller().state;                                 // 149
  var token = state.get("token");                                      // 150
  if (!token) {                                                        // 151
    return;                                                            // 152
  } else {                                                             //
    return { _token: token };                                          // 154
  }                                                                    //
};                                                                     //
                                                                       //
var handleError = function (err) {                                     // 158
  var _this2 = this;                                                   //
                                                                       //
  Meteor.setTimeout(function () {                                      // 159
    _this2.set("fadeAlert", true);                                     // 160
  }, 3000);                                                            //
                                                                       //
  if (err) {                                                           // 163
    this.set("failures", this.get("failures") + 1);                    // 164
    console.error(err);                                                // 165
    var errors = this.get("errors");                                   // 166
    errors.push(err);                                                  // 167
    this.set("errors", errors);                                        // 168
  } else {                                                             //
    this.set("successes", this.get("successes") + 1);                  // 170
  }                                                                    //
};                                                                     //
                                                                       //
var updateUser = function (options) {                                  // 174
  var state = Iron.controller().state;                                 // 175
  var token = state.get("token");                                      // 176
  resetResult(state);                                                  // 177
  state.set("successMessage", "User has been updated.");               // 178
  var handleErrorBound = handleError.bind(state);                      // 179
  Meteor.call("adminUpdateUser", token, options, handleErrorBound);    // 180
};                                                                     //
                                                                       //
var capitalize = function (str) {                                      // 183
  return str.charAt(0).toUpperCase() + str.slice(1);                   // 184
};                                                                     //
                                                                       //
if (Meteor.isClient) {                                                 // 188
  (function () {                                                       //
    AdminToken = new Mongo.Collection("adminToken"); // see Meteor.publish("adminToken")
    AdminLog = new Meteor.Collection("adminLog");                      // 190
    Meteor.subscribe("publicAdminSettings");                           // 191
                                                                       //
    Template.admin.helpers({                                           // 193
      adminTab: function () {                                          // 194
        return Router.current().route.getName();                       // 195
      },                                                               //
                                                                       //
      success: function () {                                           // 198
        var state = Iron.controller().state;                           // 199
        return state.get("successes") == state.get("numSettings");     // 200
      },                                                               //
                                                                       //
      failure: function () {                                           // 203
        return Iron.controller().state.get("failures");                // 204
      },                                                               //
                                                                       //
      errors: function () {                                            // 207
        return Iron.controller().state.get("errors");                  // 208
      },                                                               //
                                                                       //
      fadeAlert: function () {                                         // 211
        return Iron.controller().state.get("fadeAlert");               // 212
      },                                                               //
                                                                       //
      successMessage: function () {                                    // 215
        return Iron.controller().state.get("successMessage");          // 216
      },                                                               //
                                                                       //
      settingsActive: function () {                                    // 219
        return Router.current().route.getName() == "adminSettings";    // 220
      },                                                               //
                                                                       //
      usersActive: function () {                                       // 223
        return Router.current().route.getName() == "adminUsers";       // 224
      },                                                               //
                                                                       //
      invitesActive: function () {                                     // 227
        return Router.current().route.getName() == "adminInvites";     // 228
      },                                                               //
                                                                       //
      statsActive: function () {                                       // 231
        return Router.current().route.getName() == "adminStats";       // 232
      },                                                               //
                                                                       //
      logActive: function () {                                         // 235
        return Router.current().route.getName() == "adminLog";         // 236
      },                                                               //
                                                                       //
      capsActive: function () {                                        // 239
        return Router.current().route.getName() == "adminCaps";        // 240
      },                                                               //
                                                                       //
      advancedActive: function () {                                    // 243
        return Router.current().route.getName() == "adminAdvanced";    // 244
      },                                                               //
                                                                       //
      wildcardHostSeemsBroken: function () {                           // 247
        if (Session.get("alreadyTestedWildcardHost") && !Session.get("wildcardHostWorks")) {
          return true;                                                 // 250
        }                                                              //
                                                                       //
        return false;                                                  // 253
      },                                                               //
                                                                       //
      getToken: getToken                                               // 256
    });                                                                //
                                                                       //
    Template.adminSettings.events({                                    // 259
      "click .oauth-checkbox": function (event) {                      // 260
        var state = Iron.controller().state;                           // 261
        var serviceName = event.target.getAttribute("data-servicename");
        var config = Package["service-configuration"].ServiceConfiguration.configurations.findOne({ service: serviceName });
                                                                       //
        var setting = Settings.findOne({ _id: serviceName });          // 265
        if (event.target.checked && (!config || setting && setting.automaticallyReset)) {
          state.set("configurationServiceName", serviceName);          // 267
        }                                                              //
      },                                                               //
                                                                       //
      "click .configure-oauth": function (event) {                     // 271
        var state = Iron.controller().state;                           // 272
        state.set("configurationServiceName", event.target.getAttribute("data-servicename"));
      },                                                               //
                                                                       //
      "click .reset-login-tokens": function (event) {                  // 276
        var state = Iron.controller().state;                           // 277
        resetResult(state);                                            // 278
        var handleErrorBound = handleError.bind(state);                // 279
        Meteor.call("clearResumeTokensForService", this.token, event.target.getAttribute("data-servicename"), handleErrorBound);
      },                                                               //
                                                                       //
      "click #admin-settings-send-toggle": function (event) {          // 284
        var state = Iron.controller().state;                           // 285
        state.set("isEmailTestActive", !state.get("isEmailTestActive"));
        return false; // prevent form from submitting                  // 287
      },                                                               //
                                                                       //
      "click #admin-settings-send-test": function (event) {            // 290
        var state = Iron.controller().state;                           // 291
        resetResult(state);                                            // 292
        var handleErrorBound = handleError.bind(state);                // 293
        state.set("successMessage", "Email has been sent.");           // 294
        Meteor.call("testSend", this.token, document.getElementById("smptUrl").value, document.getElementById("email-test-to").value, handleErrorBound);
        return false; // prevent form from submitting                  // 297
      },                                                               //
                                                                       //
      "submit #admin-settings-form": function (event) {                // 300
        var state = Iron.controller().state;                           // 301
        var token = this.token;                                        // 302
        resetResult(state);                                            // 303
        state.set("numSettings", 4);                                   // 304
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 306
        if (event.target.emailTokenLogin.checked && !event.target.smtpUrl.value) {
          handleErrorBound(new Meteor.Error(400, "You must configure an SMTP server to use email login."));
          return false;                                                // 310
        }                                                              //
                                                                       //
        Meteor.call("setAccountSetting", token, "google", event.target.googleLogin.checked, handleErrorBound);
        Meteor.call("setAccountSetting", token, "github", event.target.githubLogin.checked, handleErrorBound);
        Meteor.call("setAccountSetting", token, "emailToken", event.target.emailTokenLogin.checked, handleErrorBound);
        Meteor.call("setSetting", token, "smtpUrl", event.target.smtpUrl.value, handleErrorBound);
        return false;                                                  // 317
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminSettings.helpers({                                   // 321
      setDocumentTitle: function () {                                  // 322
        document.title = "Settings · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      googleSetting: function () {                                     // 326
        return Settings.findOne({ _id: "google" });                    // 327
      },                                                               //
                                                                       //
      githubSetting: function () {                                     // 330
        return Settings.findOne({ _id: "github" });                    // 331
      },                                                               //
                                                                       //
      emailTokenEnabled: function () {                                 // 334
        var setting = Settings.findOne({ _id: "emailToken" });         // 335
        if (setting) {                                                 // 336
          return setting.value;                                        // 337
        } else {                                                       //
          return false;                                                // 339
        }                                                              //
      },                                                               //
                                                                       //
      smtpUrl: function () {                                           // 343
        return Iron.controller().state.get("smtpUrl");                 // 344
      },                                                               //
                                                                       //
      isEmailTestActive: function () {                                 // 347
        return Iron.controller().state.get("isEmailTestActive");       // 348
      },                                                               //
                                                                       //
      getToken: getToken                                               // 351
    });                                                                //
                                                                       //
    Template.adminUsers.onCreated(function () {                        // 354
      var state = Iron.controller().state;                             // 355
      var token = state.get("token");                                  // 356
      // TODO(perf): Paginate.                                         //
      this.subscribe("allUsers", token);                               // 358
    });                                                                //
                                                                       //
    Template.adminUsers.events({                                       // 361
      "change select.user-class": function (event) {                   // 362
        var value = event.target.selectedOptions[0].value;             // 363
                                                                       //
        if (value == "admin") {                                        // 365
          updateUser({ userId: this._id, signupKey: true, isAdmin: true });
        } else if (value == "invited") {                               //
          updateUser({ userId: this._id, signupKey: true, isAdmin: false });
        } else if (value == "guest") {                                 //
          updateUser({ userId: this._id, signupKey: false, isAdmin: false });
        } else {                                                       //
          console.error("unrecognized user class");                    // 372
        }                                                              //
      },                                                               //
                                                                       //
      "change .is-signedup-checkbox": function (event) {               // 376
        // The userid is stored on the the <tr>, which is always 2 nodes up
        var userId = event.target.parentElement.parentElement.getAttribute("data-userid");
      }                                                                //
    });                                                                //
    Template.adminUsers.helpers({                                      // 381
      setDocumentTitle: function () {                                  // 382
        document.title = "Users · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      users: function () {                                             // 386
        return Meteor.users.find({ loginIdentities: { $exists: 1 } }, { sort: { createdAt: 1 } });
      },                                                               //
                                                                       //
      userIdentity: function () {                                      // 390
        var identityId = SandstormDb.getUserIdentityIds(this)[0];      // 391
        var identity = Meteor.users.findOne({ _id: identityId });      // 392
        if (identity) {                                                // 393
          SandstormDb.fillInProfileDefaults(identity);                 // 394
          SandstormDb.fillInIntrinsicName(identity);                   // 395
          return identity;                                             // 396
        }                                                              //
      },                                                               //
                                                                       //
      userSignupNote: function () {                                    // 400
        if (this.signupEmail) {                                        // 401
          return this.signupEmail;                                     // 402
        } else if (this.signupNote) {                                  //
          return this.signupNote;                                      // 404
        } else {                                                       //
          return "";                                                   // 406
        }                                                              //
      },                                                               //
                                                                       //
      userIsAdmin: function () {                                       // 410
        return !!this.isAdmin;                                         // 411
      },                                                               //
                                                                       //
      userIsInvited: function () {                                     // 414
        return !this.isAdmin && !!this.signupKey;                      // 415
      },                                                               //
                                                                       //
      userIsGuest: function () {                                       // 418
        return !this.isAdmin && !this.signupKey;                       // 419
      },                                                               //
                                                                       //
      userStorageUsage: function () {                                  // 422
        return typeof this.storageUsage === "number" ? prettySize(this.storageUsage) : "";
      }                                                                //
    });                                                                //
                                                                       //
    var configureLoginServiceDialogTemplateForService = function (serviceName) {
      return Template["configureLoginServiceDialogFor" + capitalize(serviceName)];
    };                                                                 //
                                                                       //
    var configurationFields = function (serviceName) {                 // 431
      var template = configureLoginServiceDialogTemplateForService(serviceName);
      return template ? template.fields() : [];                        // 433
    };                                                                 //
                                                                       //
    Template._adminConfigureLoginServiceDialog.helpers({               // 436
      configurationFields: function () {                               // 437
        var serviceName = Iron.controller().state.get("configurationServiceName");
        var configurations = Package["service-configuration"].ServiceConfiguration.configurations;
        var configuration = configurations.findOne({ service: serviceName });
        var fields = configurationFields(serviceName);                 // 441
        if (configuration) {                                           // 442
          return _.map(fields, function (field) {                      // 443
            field.value = configuration[field.property];               // 444
            return field;                                              // 445
          });                                                          //
        } else {                                                       //
          return fields;                                               // 448
        }                                                              //
      },                                                               //
                                                                       //
      visible: function () {                                           // 452
        return Iron.controller().state.get("configurationServiceName") !== null;
      },                                                               //
                                                                       //
      configurationSteps: function () {                                // 456
        // renders the appropriate template                            //
        return configureLoginServiceDialogTemplateForService(Iron.controller().state.get("configurationServiceName"));
      }                                                                //
    });                                                                //
                                                                       //
    Template._adminConfigureLoginServiceDialog.events({                // 463
      "click .configure-login-service-dismiss-button": function () {   // 464
        Iron.controller().state.set("configurationServiceName", null);
      },                                                               //
                                                                       //
      "click #configure-login-service-dialog-save-configuration": function () {
        var state = Iron.controller().state;                           // 469
        resetResult(state);                                            // 470
                                                                       //
        // This is a bit of a hack, but we set the number high so that a success message is never displayed
        state.set("numSettings", 100);                                 // 473
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 475
        var serviceName = state.get("configurationServiceName");       // 476
        var token = this.token;                                        // 477
        var configuration = {                                          // 478
          service: serviceName                                         // 479
        };                                                             //
                                                                       //
        // Fetch the value of each input field                         //
        _.each(configurationFields(serviceName), function (field) {    // 483
          configuration[field.property] = document.getElementById("configure-login-service-dialog-" + field.property).value.trim();
        });                                                            //
                                                                       //
        configuration.loginStyle = "redirect";                         // 488
                                                                       //
        Meteor.call("adminConfigureLoginService", token, configuration, function (err) {
          handleErrorBound(err);                                       // 491
          state.set("configurationServiceName", null);                 // 492
        });                                                            //
                                                                       //
        Meteor.call("setAccountSetting", token, serviceName, true, handleErrorBound);
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminInvites.events({                                     // 499
      "click #send": function (event) {                                // 500
        var state = Iron.controller().state;                           // 501
        var from = document.getElementById("invite-from").value;       // 502
        var list = document.getElementById("invite-emails").value;     // 503
        var subject = document.getElementById("invite-subject").value;
        var message = document.getElementById("invite-message").value;
        var quotaInput = document.getElementById("invite-quota");      // 506
        var quota = undefined;                                         // 507
        if (quotaInput && quotaInput.value.trim() !== "") {            // 508
          quota = parseInt(quotaInput.value);                          // 509
        }                                                              //
                                                                       //
        var sendButton = event.currentTarget;                          // 512
        sendButton.disabled = true;                                    // 513
        var oldContent = sendButton.textContent;                       // 514
        sendButton.textContent = "Sending...";                         // 515
                                                                       //
        Meteor.call("sendInvites", state.get("token"), getOrigin(), from, list, subject, message, quota, function (error, results) {
          sendButton.disabled = false;                                 // 519
          sendButton.textContent = oldContent;                         // 520
          if (error) {                                                 // 521
            state.set("inviteMessage", { error: error.toString() });   // 522
          } else {                                                     //
            state.set("inviteMessage", results);                       // 524
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #create": function (event) {                              // 529
        var state = Iron.controller().state;                           // 530
        var note = document.getElementById("key-note").value;          // 531
        var quotaInput = document.getElementById("key-quota");         // 532
        var quota = undefined;                                         // 533
        if (quotaInput && quotaInput.value.trim() !== "") {            // 534
          quota = parseInt(quotaInput.value);                          // 535
        }                                                              //
                                                                       //
        Meteor.call("createSignupKey", state.get("token"), note, quota, function (error, key) {
          if (error) {                                                 // 539
            state.set("inviteMessage", { error: error.toString() });   // 540
          } else {                                                     //
            state.set("inviteMessage", {                               // 542
              url: getOrigin() + Router.routes.signup.path({ key: key })
            });                                                        //
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click #set-quota-submit": function (event) {                    // 549
        var state = Iron.controller().state;                           // 550
        var list = document.getElementById("set-quota-emails").value;  // 551
        var quotaInput = document.getElementById("set-quota-quota");   // 552
        var quota = undefined;                                         // 553
        if (quotaInput && quotaInput.value.trim() !== "") {            // 554
          quota = parseInt(quotaInput.value);                          // 555
        }                                                              //
                                                                       //
        var updateButton = event.currentTarget;                        // 558
        updateButton.disabled = true;                                  // 559
        var oldContent = updateButton.textContent;                     // 560
        updateButton.textContent = "Updating...";                      // 561
                                                                       //
        Meteor.call("updateQuotas", state.get("token"), list, quota, function (error, results) {
          updateButton.disabled = false;                               // 564
          updateButton.textContent = oldContent;                       // 565
          if (error) {                                                 // 566
            state.set("inviteMessage", { error: error.toString() });   // 567
          } else {                                                     //
            document.getElementById("set-quota-emails").value = "";    // 569
          }                                                            //
        });                                                            //
      },                                                               //
                                                                       //
      "click .autoSelect": function (event) {                          // 574
        event.currentTarget.select();                                  // 575
      },                                                               //
                                                                       //
      "click #retry": function (event) {                               // 578
        Iron.controller().state.set("inviteMessage", undefined);       // 579
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminInvites.helpers({                                    // 583
      setDocumentTitle: function () {                                  // 584
        document.title = "Invites · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      error: function () {                                             // 588
        var res = Iron.controller().state.get("inviteMessage");        // 589
        return res && res.error;                                       // 590
      },                                                               //
                                                                       //
      email: function () {                                             // 593
        var me = Meteor.user();                                        // 594
        var email = me.services && me.services.google && me.services.google.email || me.services && me.services.github && me.services.github.email;
        if (email && me.profile.name) {                                // 597
          email = me.profile.name + " <" + email + ">";                // 598
        }                                                              //
                                                                       //
        email = email || "";                                           // 601
        return email;                                                  // 602
      },                                                               //
                                                                       //
      url: function () {                                               // 605
        var res = Iron.controller().state.get("inviteMessage");        // 606
        return res && res.url;                                         // 607
      },                                                               //
                                                                       //
      sent: function () {                                              // 610
        var res = Iron.controller().state.get("inviteMessage");        // 611
        return res && res.sent;                                        // 612
      }                                                                //
    });                                                                //
                                                                       //
    var maybeScrollLog = function () {                                 // 616
      // This function appears unused.  TODO: verify safe to delete    //
      var elem = document.getElementById("adminLog");                  // 618
      var scrollLogToBottom = function (elem) {                        // 619
        elem.scrollTop = elem.scrollHeight;                            // 620
      };                                                               //
                                                                       //
      if (elem) {                                                      // 623
        // The log already exists. It's about to be updated. Check if it's scrolled to the bottom
        // before the update.                                          //
        if (elem.scrollHeight - elem.scrollTop === elem.clientHeight) {
          // Indeed, so we want to scroll it back to the bottom after the update.
          Tracker.afterFlush(function () {                             // 628
            scrollLogToBottom(elem);                                   // 628
          });                                                          //
        }                                                              //
      } else {                                                         //
        // No element exists yet, but it's probably about to be created, in which case we definitely
        // want to scroll it.                                          //
        Tracker.afterFlush(function () {                               // 633
          var elem2 = document.getElementById("adminLog");             // 634
          if (elem2) scrollLogToBottom(elem2);                         // 635
        });                                                            //
      }                                                                //
    };                                                                 //
                                                                       //
    Template.adminLog.onCreated(function () {                          // 640
      var state = Iron.controller().state;                             // 641
      var token = state.get("token");                                  // 642
      this.subscribe("adminLog", token);                               // 643
    });                                                                //
                                                                       //
    Template.adminLog.helpers({                                        // 646
      setDocumentTitle: function () {                                  // 647
        document.title = "Log · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      html: function () {                                              // 651
        // jscs:disable requireCamelCaseOrUpperCaseIdentifiers         //
        return AnsiUp.ansi_to_html(AdminLog.find({}, { $sort: { _id: 1 } }).map(function (entry) {
          return entry.text;                                           // 654
        }).join(""), { use_classes: true });                           //
        // jscs:enable requireCamelCaseOrUpperCaseIdentifiers          //
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminCaps.onCreated(function () {                         // 660
      var state = Iron.controller().state;                             // 661
      var token = state.get("token");                                  // 662
      this.subscribe("adminApiTokens", token);                         // 663
                                                                       //
      // TODO(perf): Don't subscribe to all users, just to the ones that are referenced from
      //   the relevant ApiTokens.                                     //
      this.subscribe("allUsers", token);                               // 667
    });                                                                //
                                                                       //
    Template.adminCaps.helpers({                                       // 670
      setDocumentTitle: function () {                                  // 671
        document.title = "Capabilities · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      showPowerboxOffer: function () {                                 // 675
        var state = Iron.controller().state;                           // 676
        return !!state.get("powerboxOfferUrl");                        // 677
      },                                                               //
                                                                       //
      powerboxOfferData: function () {                                 // 680
        var state = Iron.controller().state;                           // 681
        return {                                                       // 682
          get: function () {                                           // 683
            return {                                                   // 684
              offer: {                                                 // 685
                url: state.get("powerboxOfferUrl")                     // 686
              },                                                       //
              onDismiss: function () {                                 // 688
                state.set("powerboxOfferUrl", null);                   // 689
              }                                                        //
            };                                                         //
          }                                                            //
        };                                                             //
      },                                                               //
                                                                       //
      caps: function () {                                              // 696
        return ApiTokens.find({                                        // 697
          $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
        });                                                            //
      },                                                               //
                                                                       //
      userName: function () {                                          // 705
        var userId = findAdminUserForToken(this);                      // 706
        var user = Meteor.users.findOne({ _id: userId });              // 707
        if (!user) {                                                   // 708
          return "no user";                                            // 709
        }                                                              //
                                                                       //
        var identityId = SandstormDb.getUserIdentityIds(user)[0];      // 712
        var identity = Meteor.users.findOne({ _id: identityId });      // 713
        if (identity) {                                                // 714
          SandstormDb.fillInProfileDefaults(identity);                 // 715
          return identity.profile.name;                                // 716
        }                                                              //
      },                                                               //
                                                                       //
      isDisabled: function () {                                        // 720
        return !this.userId;                                           // 721
      },                                                               //
                                                                       //
      disabled: function () {                                          // 724
        return this.revoked;                                           // 725
      }                                                                //
    });                                                                //
                                                                       //
    var updateCap = function (capId, value) {                          // 729
      var state = Iron.controller().state;                             // 730
      resetResult(state);                                              // 731
      if (!value) {                                                    // 732
        state.set("successMessage", "Capability has been re-enabled.");
      } else {                                                         //
        state.set("successMessage", "Capability has been disabled.");  // 735
      }                                                                //
                                                                       //
      var handleErrorBound = handleError.bind(state);                  // 738
      Meteor.call("adminToggleDisableCap", state.get("token"), capId, value, handleErrorBound);
    };                                                                 //
                                                                       //
    Template.adminCaps.events({                                        // 742
      "click #offer-ipnetwork": function (event) {                     // 743
        var state = Iron.controller().state;                           // 744
        resetResult(state);                                            // 745
        state.set("successMessage", "IpNetwork webkey created. Look for it in the top bar.");
        Meteor.call("offerIpNetwork", this.token, function (err, webkey) {
          state.set("powerboxOfferUrl", webkey);                       // 748
          handleError.call(state, err);                                // 749
        });                                                            //
                                                                       //
        return false; // prevent form from submitting                  // 752
      },                                                               //
                                                                       //
      "click #offer-ipinterface": function (event) {                   // 755
        var state = Iron.controller().state;                           // 756
        resetResult(state);                                            // 757
        state.set("successMessage", "IpInterface webkey created. Look for it in the top bar.");
        Meteor.call("offerIpInterface", this.token, function (err, webkey) {
          state.set("powerboxOfferUrl", webkey);                       // 760
          handleError.call(state, err);                                // 761
        });                                                            //
                                                                       //
        return false; // prevent form from submitting                  // 764
      },                                                               //
                                                                       //
      "click #powerbox-offer-popup-closer": function (event) {         // 767
        var state = Iron.controller().state;                           // 768
        return state.set("powerboxOfferUrl", null);                    // 769
      },                                                               //
                                                                       //
      "click .disable-cap": function (event) {                         // 772
        var capId = event.target.getAttribute("data-id");              // 773
        var token = ApiTokens.findOne({ _id: capId });                 // 774
                                                                       //
        updateCap(capId, !token.revoked);                              // 776
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminAdvanced.events({                                    // 780
      "submit #admin-settings-form": function (event) {                // 781
        var state = Iron.controller().state;                           // 782
        var token = this.token;                                        // 783
        resetResult(state);                                            // 784
        state.set("numSettings", 12);                                  // 785
                                                                       //
        var handleErrorBound = handleError.bind(state);                // 787
        Meteor.call("setSetting", token, "serverTitle", event.target.serverTitle.value, handleErrorBound);
        Meteor.call("setSetting", token, "returnAddress", event.target.returnAddress.value, handleErrorBound);
        Meteor.call("setSetting", token, "splashUrl", event.target.splashUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "signupDialog", event.target.signupDialog.value, handleErrorBound);
        Meteor.call("setSetting", token, "termsUrl", event.target.termsUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "privacyUrl", event.target.privacyUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "adminAlert", event.target.adminAlert.value, handleErrorBound);
        Meteor.call("setSetting", token, "appMarketUrl", event.target.appMarketUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "appIndexUrl", event.target.appIndexUrl.value, handleErrorBound);
        Meteor.call("setSetting", token, "appUpdatesEnabled", event.target.appUpdatesEnabled.checked, handleErrorBound);
        var alertTimeString = event.target.alertTime.value.trim();     // 800
        if (alertTimeString) {                                         // 801
          var alertTime = new Date(alertTimeString);                   // 802
          if (isNaN(alertTime.getTime())) {                            // 803
            // Assume only time and not date was set.                  //
            alertTime = new Date(new Date().toLocaleDateString() + " " + alertTimeString);
          }                                                            //
                                                                       //
          if (isNaN(alertTime.getTime())) {                            // 808
            handleErrorBound(new Meteor.Error(400, "Couldn't parse alert time, please be more precise."));
          } else {                                                     //
            Meteor.call("setSetting", token, "adminAlertTime", alertTime, handleErrorBound);
          }                                                            //
        } else {                                                       //
          Meteor.call("setSetting", token, "adminAlertTime", null, handleErrorBound);
        }                                                              //
                                                                       //
        Meteor.call("setSetting", token, "adminAlertUrl", event.target.alertUrl.value, handleErrorBound);
        return false;                                                  // 819
      }                                                                //
    });                                                                //
                                                                       //
    Template.adminAdvanced.helpers({                                   // 823
      setDocumentTitle: function () {                                  // 824
        document.title = "Advanced · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      serverTitle: function () {                                       // 828
        var setting = Settings.findOne({ _id: "serverTitle" });        // 829
        return setting && setting.value || "";                         // 830
      },                                                               //
                                                                       //
      returnAddress: function () {                                     // 833
        var setting = Settings.findOne({ _id: "returnAddress" });      // 834
        return setting && setting.value || "";                         // 835
      },                                                               //
                                                                       //
      splashUrl: function () {                                         // 838
        var setting = Settings.findOne({ _id: "splashUrl" });          // 839
        return setting && setting.value || "";                         // 840
      },                                                               //
                                                                       //
      origin: function () {                                            // 843
        return getOrigin();                                            // 843
      },                                                               //
                                                                       //
      signupDialog: function () {                                      // 845
        var setting = Settings.findOne({ _id: "signupDialog" });       // 846
        return setting && setting.value || DEFAULT_SIGNUP_DIALOG;      // 847
      },                                                               //
                                                                       //
      termsUrl: function () {                                          // 850
        var setting = Settings.findOne({ _id: "termsUrl" });           // 851
        return setting && setting.value;                               // 852
      },                                                               //
                                                                       //
      privacyUrl: function () {                                        // 855
        var setting = Settings.findOne({ _id: "privacyUrl" });         // 856
        return setting && setting.value;                               // 857
      },                                                               //
                                                                       //
      adminAlert: function () {                                        // 860
        var setting = Settings.findOne({ _id: "adminAlert" });         // 861
        return setting && setting.value;                               // 862
      },                                                               //
                                                                       //
      alertTime: function () {                                         // 865
        var setting = Settings.findOne({ _id: "adminAlertTime" });     // 866
        if (setting && setting.value) {                                // 867
          return setting.value.toLocaleDateString() + " " + setting.value.toLocaleTimeString();
        } else {                                                       //
          return "";                                                   // 870
        }                                                              //
      },                                                               //
                                                                       //
      alertUrl: function () {                                          // 874
        var setting = Settings.findOne({ _id: "adminAlertUrl" });      // 875
        return setting && setting.value;                               // 876
      },                                                               //
                                                                       //
      appMarketUrl: function () {                                      // 879
        var setting = Settings.findOne({ _id: "appMarketUrl" });       // 880
        return setting && setting.value;                               // 881
      },                                                               //
                                                                       //
      appIndexUrl: function () {                                       // 884
        var setting = Settings.findOne({ _id: "appIndexUrl" });        // 885
        return setting && setting.value;                               // 886
      },                                                               //
                                                                       //
      appUpdatesEnabled: function () {                                 // 889
        var setting = Settings.findOne({ _id: "appUpdatesEnabled" });  // 890
        return setting && setting.value;                               // 891
      }                                                                //
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
if (Meteor.isServer) {                                                 // 896
  (function () {                                                       //
    var Fs = Npm.require("fs");                                        // 897
    var SANDSTORM_ADMIN_TOKEN = SANDSTORM_VARDIR + "/adminToken";      // 898
                                                                       //
    var getSmtpUrl = function () {                                     // 900
      var setting = Settings.findOne({ _id: "smtpUrl" });              // 901
      if (setting) {                                                   // 902
        return setting.value;                                          // 903
      } else {                                                         //
        return process.env.MAIL_URL;                                   // 905
      }                                                                //
    };                                                                 //
                                                                       //
    var tokenIsValid = function (token) {                              // 909
      if (token && Fs.existsSync(SANDSTORM_ADMIN_TOKEN)) {             // 910
        var stats = Fs.statSync(SANDSTORM_ADMIN_TOKEN);                // 911
        var expireTime = new Date(Date.now() - ADMIN_TOKEN_EXPIRATION_TIME);
        if (stats.mtime < expireTime) {                                // 913
          return false;                                                // 914
        } else {                                                       //
          return Fs.readFileSync(SANDSTORM_ADMIN_TOKEN, { encoding: "utf8" }) === token;
        }                                                              //
      } else {                                                         //
        return false;                                                  // 919
      }                                                                //
    };                                                                 //
                                                                       //
    var checkAuth = function (token) {                                 // 923
      check(token, Match.OneOf(undefined, null, String));              // 924
      if (!isAdmin() && !tokenIsValid(token)) {                        // 925
        throw new Meteor.Error(403, "User must be admin or provide a valid token");
      }                                                                //
    };                                                                 //
                                                                       //
    var clearAdminToken = function (token) {                           // 930
      if (tokenIsValid(token)) {                                       // 931
        Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                          // 932
        console.log("Admin token deleted.");                           // 933
      }                                                                //
    };                                                                 //
                                                                       //
    Meteor.methods({                                                   // 937
      setAccountSetting: function (token, serviceName, value) {        // 938
        checkAuth(token);                                              // 939
        check(serviceName, String);                                    // 940
        check(value, Boolean);                                         // 941
                                                                       //
        // TODO(someday): currently this relies on the fact that an account is tied to a single
        // identity, and thus has only that entry in "services". This will need to be looked at when
        // multiple login methods/identities are allowed for a single account.
        if (!value && !tokenIsValid(token) && serviceName in Meteor.user().services) {
          throw new Meteor.Error(403, "You can not disable the login service that your account uses.");
        }                                                              //
                                                                       //
        // Only check configurations for OAuth services.               //
        var oauthServices = ["google", "github"];                      // 952
        if (value && oauthServices.indexOf(serviceName) != -1) {       // 953
          var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
          var config = ServiceConfiguration.configurations.findOne({ service: serviceName });
          if (!config) {                                               // 956
            throw new Meteor.Error(403, "You must configure the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
          }                                                            //
                                                                       //
          if (!config.clientId || !config.secret) {                    // 961
            throw new Meteor.Error(403, "You must provide a non-empty clientId and secret for the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
          }                                                            //
        }                                                              //
                                                                       //
        Settings.upsert({ _id: serviceName }, { $set: { value: value } });
        if (value) {                                                   // 968
          Settings.update({ _id: serviceName }, { $unset: { automaticallyReset: 1 } });
        }                                                              //
      },                                                               //
                                                                       //
      setSetting: function (token, name, value) {                      // 973
        checkAuth(token);                                              // 974
        check(name, String);                                           // 975
        check(value, Match.OneOf(null, String, Date, Boolean));        // 976
                                                                       //
        Settings.upsert({ _id: name }, { $set: { value: value } });    // 978
      },                                                               //
                                                                       //
      getSmtpUrl: function (token) {                                   // 981
        checkAuth(token);                                              // 982
                                                                       //
        return getSmtpUrl();                                           // 984
      },                                                               //
                                                                       //
      adminConfigureLoginService: function (token, options) {          // 987
        checkAuth(token);                                              // 988
        check(options, Match.ObjectIncluding({ service: String }));    // 989
                                                                       //
        var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
                                                                       //
        ServiceConfiguration.configurations.upsert({ service: options.service }, options);
      },                                                               //
                                                                       //
      clearResumeTokensForService: function (token, serviceName) {     // 996
        checkAuth(token);                                              // 997
        check(serviceName, String);                                    // 998
                                                                       //
        var query = {};                                                // 1000
        query["services." + serviceName] = { $exists: true };          // 1001
        Meteor.users.find(query).forEach(function (identity) {         // 1002
          if (identity.services.resume && identity.services.resume.loginTokens && identity.services.resume.loginTokens.length > 0) {
            Meteor.users.update({ _id: identity._id }, { $set: { "services.resume.loginTokens": [] } });
          }                                                            //
                                                                       //
          Meteor.users.update({ "loginIdentities.id": identity._id }, { $set: { "services.resume.loginTokens": [] } });
        });                                                            //
      },                                                               //
                                                                       //
      adminUpdateUser: function (token, userInfo) {                    // 1013
        checkAuth(token);                                              // 1014
        check(userInfo, {                                              // 1015
          userId: String,                                              // 1016
          signupKey: Boolean,                                          // 1017
          isAdmin: Boolean                                             // 1018
        });                                                            //
                                                                       //
        var userId = userInfo.userId;                                  // 1021
        if (userId === Meteor.userId() && !userInfo.isAdmin) {         // 1022
          throw new Meteor.Error(403, "User cannot remove admin permissions from itself.");
        }                                                              //
                                                                       //
        Meteor.users.update({ _id: userId }, { $set: _.omit(userInfo, ["_id", "userId"]) });
      },                                                               //
                                                                       //
      testSend: function (token, smtpUrl, to) {                        // 1029
        checkAuth(token);                                              // 1030
        check(smtpUrl, String);                                        // 1031
        check(to, String);                                             // 1032
                                                                       //
        SandstormEmail.send({                                          // 1034
          to: to,                                                      // 1035
          from: globalDb.getServerTitle() + " <" + globalDb.getReturnAddress() + ">",
          subject: "Testing your Sandstorm's SMTP setting",            // 1037
          text: "Success! Your outgoing SMTP is working.",             // 1038
          smtpUrl: smtpUrl                                             // 1039
        });                                                            //
      },                                                               //
                                                                       //
      createSignupKey: function (token, note, quota) {                 // 1043
        checkAuth(token);                                              // 1044
        check(note, String);                                           // 1045
        check(quota, Match.OneOf(undefined, null, Number));            // 1046
                                                                       //
        var key = Random.id();                                         // 1048
        var content = { _id: key, used: false, note: note };           // 1049
        if (typeof quota === "number") content.quota = quota;          // 1050
        SignupKeys.insert(content);                                    // 1051
        return key;                                                    // 1052
      },                                                               //
                                                                       //
      sendInvites: function (token, origin, from, list, subject, message, quota) {
        checkAuth(token);                                              // 1056
        check([origin, from, list, subject, message], [String]);       // 1057
        check(quota, Match.OneOf(undefined, null, Number));            // 1058
                                                                       //
        if (!from.trim()) {                                            // 1060
          throw new Meteor.Error(403, "Must enter 'from' address.");   // 1061
        }                                                              //
                                                                       //
        if (!list.trim()) {                                            // 1064
          throw new Meteor.Error(403, "Must enter 'to' addresses.");   // 1065
        }                                                              //
                                                                       //
        this.unblock();                                                // 1068
                                                                       //
        list = list.split("\n");                                       // 1070
        for (var i in babelHelpers.sanitizeForInObject(list)) {        // 1071
          var email = list[i].trim();                                  // 1072
                                                                       //
          if (email) {                                                 // 1074
            var key = Random.id();                                     // 1075
                                                                       //
            var content = {                                            // 1077
              _id: key,                                                // 1078
              used: false,                                             // 1079
              note: "E-mail invite to " + email,                       // 1080
              email: email,                                            // 1081
              definitelySent: false                                    // 1082
            };                                                         //
            if (typeof quota === "number") content.quota = quota;      // 1084
            SignupKeys.insert(content);                                // 1085
            SandstormEmail.send({                                      // 1086
              to: email,                                               // 1087
              from: from,                                              // 1088
              subject: subject,                                        // 1089
              text: message.replace(/\$KEY/g, origin + Router.routes.signup.path({ key: key }))
            });                                                        //
            SignupKeys.update(key, { $set: { definitelySent: true } });
          }                                                            //
        }                                                              //
                                                                       //
        return { sent: true };                                         // 1096
      },                                                               //
                                                                       //
      offerIpNetwork: function (token) {                               // 1099
        checkAuth(token);                                              // 1100
        if (!isAdmin()) {                                              // 1101
          throw new Meteor.Error(403, "Offering IpNetwork is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
        }                                                              //
                                                                       //
        var requirements = [{ userIsAdmin: Meteor.userId() }];         // 1106
        var sturdyRef = waitPromise(saveFrontendRef({ ipNetwork: true }, { webkey: null }, requirements)).sturdyRef;
        return ROOT_URL.protocol + "//" + makeWildcardHost("api") + "#" + sturdyRef;
      },                                                               //
                                                                       //
      offerIpInterface: function (token) {                             // 1115
        checkAuth(token);                                              // 1116
        if (!isAdmin()) {                                              // 1117
          throw new Meteor.Error(403, "Offering IpInterface is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
        }                                                              //
                                                                       //
        var requirements = [{ userIsAdmin: Meteor.userId() }];         // 1122
        var sturdyRef = waitPromise(saveFrontendRef({ ipInterface: true }, { webkey: null }, requirements)).sturdyRef;
        return ROOT_URL.protocol + "//" + makeWildcardHost("api") + "#" + sturdyRef;
      },                                                               //
                                                                       //
      adminToggleDisableCap: function (token, capId, value) {          // 1131
        checkAuth(token);                                              // 1132
        check(capId, String);                                          // 1133
        check(value, Boolean);                                         // 1134
                                                                       //
        if (value) {                                                   // 1136
          ApiTokens.update({ _id: capId }, { $set: { revoked: true } });
        } else {                                                       //
          ApiTokens.update({ _id: capId }, { $set: { revoked: false } });
        }                                                              //
      },                                                               //
                                                                       //
      updateQuotas: function (token, list, quota) {                    // 1143
        checkAuth(token);                                              // 1144
        check(list, String);                                           // 1145
        check(quota, Match.OneOf(undefined, null, Number));            // 1146
                                                                       //
        if (!list.trim()) {                                            // 1148
          throw new Meteor.Error(400, "Must enter addresses.");        // 1149
        }                                                              //
                                                                       //
        var items = list.split("\n");                                  // 1152
        var invalid = [];                                              // 1153
        for (var i in babelHelpers.sanitizeForInObject(items)) {       // 1154
          var modifier = typeof quota === "number" ? { $set: { quota: quota } } : { $unset: { quota: "" } };
          var n = SignupKeys.update({ email: items[i] }, modifier, { multi: true });
          n += Meteor.users.update({ signupEmail: items[i] }, modifier, { multi: true });
                                                                       //
          if (n < 1) invalid.push(items[i]);                           // 1160
        }                                                              //
                                                                       //
        if (invalid.length > 0) {                                      // 1163
          throw new Meteor.Error(404, "These addresses did not map to any user nor invite: " + invalid.join(", "));
        }                                                              //
      },                                                               //
                                                                       //
      dismissAdminStatsNotifications: function (token) {               // 1169
        checkAuth(token);                                              // 1170
        globalDb.collections.notifications.remove({ "admin.type": "reportStats" });
      },                                                               //
                                                                       //
      signUpAsAdmin: function (token) {                                // 1174
        check(token, String);                                          // 1175
        checkAuth(token);                                              // 1176
        if (!this.userId) {                                            // 1177
          throw new Meteor.Error(403, "Must be logged in to sign up as admin.");
        }                                                              //
                                                                       //
        if (!Meteor.user().loginIdentities) {                          // 1181
          throw new Meteor.Error(403, "Must be logged into an account to sign up as admin.");
        }                                                              //
                                                                       //
        Meteor.users.update({ _id: this.userId }, { $set: { isAdmin: true, signupKey: "admin" } });
        clearAdminToken(token);                                        // 1186
      }                                                                //
    });                                                                //
                                                                       //
    var authorizedAsAdmin = function (token, userId) {                 // 1190
      return Match.test(token, Match.OneOf(undefined, null, String)) && (userId && isAdminById(userId) || tokenIsValid(token));
    };                                                                 //
                                                                       //
    Meteor.publish("admin", function (token) {                         // 1195
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1196
      return Settings.find();                                          // 1197
    });                                                                //
                                                                       //
    Meteor.publish("adminServiceConfiguration", function (token) {     // 1200
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1201
      return Package["service-configuration"].ServiceConfiguration.configurations.find();
    });                                                                //
                                                                       //
    Meteor.publish("publicAdminSettings", function () {                // 1205
      return Settings.find({ _id: { $in: publicAdminSettings } });     // 1206
    });                                                                //
                                                                       //
    Meteor.publish("adminToken", function (token) {                    // 1209
      check(token, String);                                            // 1210
      this.added("adminToken", "adminToken", { tokenIsValid: tokenIsValid(token) });
      this.ready();                                                    // 1212
    });                                                                //
                                                                       //
    Meteor.publish("allUsers", function (token) {                      // 1215
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1216
      return Meteor.users.find();                                      // 1217
    });                                                                //
                                                                       //
    Meteor.publish("activityStats", function (token) {                 // 1220
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1221
      return ActivityStats.find();                                     // 1222
    });                                                                //
                                                                       //
    Meteor.publish("statsTokens", function (token) {                   // 1225
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1226
      return StatsTokens.find();                                       // 1227
    });                                                                //
                                                                       //
    Meteor.publish("allPackages", function (token) {                   // 1230
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1231
      return Packages.find({ manifest: { $exists: true } }, { fields: { appId: 1, "manifest.appVersion": 1,
          "manifest.actions": 1, "manifest.appTitle": 1 } });          // 1234
    });                                                                //
                                                                       //
    Meteor.publish("realTimeStats", function (token) {                 // 1237
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1238
                                                                       //
      // Last five minutes.                                            //
      this.added("realTimeStats", "now", computeStats(new Date(Date.now() - 5 * 60 * 1000)));
                                                                       //
      // Since last sample.                                            //
      var lastSample = ActivityStats.findOne({}, { sort: { timestamp: -1 } });
      var lastSampleTime = lastSample ? lastSample.timestamp : new Date(0);
      this.added("realTimeStats", "today", computeStats(lastSampleTime));
                                                                       //
      // TODO(someday): Update every few minutes?                      //
                                                                       //
      this.ready();                                                    // 1250
    });                                                                //
                                                                       //
    Meteor.publish("adminLog", function (token) {                      // 1253
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1254
                                                                       //
      var logfile = SANDSTORM_LOGDIR + "/sandstorm.log";               // 1256
                                                                       //
      var fd = Fs.openSync(logfile, "r");                              // 1258
      var startSize = Fs.fstatSync(fd).size;                           // 1259
                                                                       //
      // Start tailing at EOF - 8k.                                    //
      var offset = Math.max(0, startSize - 8192);                      // 1262
                                                                       //
      var _this = this;                                                // 1264
      function doTail() {                                              // 1265
        for (;;) {                                                     // 1266
          var buf = new Buffer(Math.max(1024, startSize - offset));    // 1267
          var n = Fs.readSync(fd, buf, 0, buf.length, offset);         // 1268
          if (n <= 0) break;                                           // 1269
          _this.added("adminLog", offset, { text: buf.toString("utf8", 0, n) });
          offset += n;                                                 // 1271
        }                                                              //
      }                                                                //
                                                                       //
      // Watch the file for changes.                                   //
      var watcher = Fs.watch(logfile, { persistent: false }, Meteor.bindEnvironment(doTail));
                                                                       //
      // When the subscription stops, stop watching the file.          //
      this.onStop(function () {                                        // 1279
        watcher.close();                                               // 1280
        Fs.closeSync(fd);                                              // 1281
      });                                                              //
                                                                       //
      // Read initial 8k tail data immediately.                        //
      doTail();                                                        // 1285
                                                                       //
      // Notify ready.                                                 //
      this.ready();                                                    // 1288
    });                                                                //
                                                                       //
    Meteor.publish("adminApiTokens", function (token) {                // 1291
      if (!authorizedAsAdmin(token, this.userId)) return [];           // 1292
      return ApiTokens.find({                                          // 1293
        $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
      }, {                                                             //
        fields: {                                                      // 1299
          frontendRef: 1,                                              // 1300
          created: 1,                                                  // 1301
          requirements: 1,                                             // 1302
          revoked: 1                                                   // 1303
        }                                                              //
      });                                                              //
    });                                                                //
  })();                                                                //
}                                                                      //
                                                                       //
function serviceEnabled(name) {                                        // 1309
  var setting = Settings.findOne({ _id: name });                       // 1310
  return setting && !!setting.value;                                   // 1311
}                                                                      //
                                                                       //
if (Meteor.isServer) {                                                 // 1314
  function observeOauthService(name) {                                 // 1315
    Settings.find({ _id: name, value: true }).observe({                // 1316
      added: function () {                                             // 1317
        // Tell the oauth library it should accept login attempts from this service.
        Accounts.oauth.registerService(name);                          // 1319
      },                                                               //
                                                                       //
      removed: function () {                                           // 1322
        // Tell the oauth library it should deny login attempts from this service.
        Accounts.oauth.unregisterService(name);                        // 1324
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  observeOauthService("github");                                       // 1329
  observeOauthService("google");                                       // 1330
}                                                                      //
                                                                       //
Accounts.identityServices.github = {                                   // 1333
  isEnabled: function () {                                             // 1334
    return serviceEnabled("github");                                   // 1335
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1338
    name: "oauthLoginButton",                                          // 1339
    priority: 1,                                                       // 1340
    data: {                                                            // 1341
      method: "loginWithGithub",                                       // 1342
      name: "github",                                                  // 1343
      displayName: "GitHub"                                            // 1344
    }                                                                  //
  }                                                                    //
};                                                                     //
                                                                       //
Accounts.identityServices.google = {                                   // 1349
  isEnabled: function () {                                             // 1350
    return serviceEnabled("google");                                   // 1351
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1354
    name: "oauthLoginButton",                                          // 1355
    priority: 2,                                                       // 1356
    data: {                                                            // 1357
      method: "loginWithGoogle",                                       // 1358
      name: "google",                                                  // 1359
      displayName: "Google"                                            // 1360
    }                                                                  //
  }                                                                    //
};                                                                     //
                                                                       //
Accounts.identityServices.email = {                                    // 1365
  isEnabled: function () {                                             // 1366
    return serviceEnabled("emailToken");                               // 1367
  },                                                                   //
                                                                       //
  loginTemplate: {                                                     // 1370
    name: "emailLoginForm",                                            // 1371
    priority: 10 }                                                     // 1372
};                                                                     //
// Put it at the bottom of the list.                                   //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=admin.js.map
