(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/stats.js                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var DAY_MS = 24 * 60 * 60 * 1000;                                      // 17
                                                                       //
if (Meteor.isServer) {                                                 // 19
  if (Mongo.Collection.prototype.aggregate) {                          // 20
    throw new Error("Looks like Meteor wrapped the Collection.aggregate() call. Make sure it " + "works then delete our own wrapper.");
  }                                                                    //
                                                                       //
  Mongo.Collection.prototype.aggregate = function () {                 // 25
    // Meteor doesn't wrapp Mongo's aggregate() method.                //
    var raw = this.rawCollection();                                    // 27
    return Meteor.wrapAsync(raw.aggregate, raw).apply(raw, arguments);
  };                                                                   //
                                                                       //
  computeStats = function (since) {                                    // 31
    // We'll need this for a variety of queries.                       //
    var timeConstraint = { $gt: since };                               // 33
                                                                       //
    // This calculates the number of user accounts that have been used
    // during the requested time period.                               //
    var currentlyActiveUsersCount = Meteor.users.find({ expires: { $exists: false }, loginIdentities: { $exists: true },
      lastActive: timeConstraint }).count();                           // 39
                                                                       //
    // This calculates the number of grains that have been used during
    // the requested time period.                                      //
    var activeGrainsCount = Grains.find({ lastUsed: timeConstraint }).count();
                                                                       //
    // If Meteor.settings.allowDemoAccounts is true, DeleteStats       //
    // contains records of type `user` and `appDemoUser`, indicating   //
    // the number of those types of accounts that were created and     //
    // then auto-expired through the demo mode's auto-account-expiry.  //
    var deletedDemoUsersCount = DeleteStats.find({ type: "demoUser", lastActive: timeConstraint }).count();
    var deletedAppDemoUsersCount = DeleteStats.find({ type: "appDemoUser", lastActive: timeConstraint }).count();
                                                                       //
    // Similarly, if the demo is enabled, we auto-delete grains; we store that
    // fact in DeleteStats with type: "grain".                         //
    var deletedGrainsCount = DeleteStats.find({ type: "grain", lastActive: timeConstraint }).count();
                                                                       //
    var apps = Grains.aggregate([{ $match: { lastUsed: timeConstraint } }, { $group: {
        _id: "$appId",                                                 // 62
        grains: { $sum: 1 },                                           // 63
        userIds: { $addToSet: "$userId" }                              // 64
      }                                                                //
    }, { $project: {                                                   //
        grains: 1,                                                     // 68
        owners: { $size: "$userIds" }                                  // 69
      }                                                                //
    }]);                                                               //
    apps = _.indexBy(apps, "_id");                                     // 73
                                                                       //
    for (var appId in babelHelpers.sanitizeForInObject(apps)) {        // 75
      // We need to count ApiTokens, which don't have appId denormalized into them. We therefore
      // have to fetch a list of grainIds first.                       //
      // TODO(perf): If stats are getting slow, denormalize appId into ApiTokens. Note that it is
      //   already denormalized for the specific case of apps that don't have icons, but the data
      //   for that use case is NOT safe to use here because it's intended that we might allow an
      //   app to mimic another app's icon by spoofing that app ID. In other words, the existing
      //   denormalization of appId should be considered "app ID for identicon purposes only". We'll
      //   need to add a new denormalization for stats purposes -- and make sure that it is not
      //   revealed to the user.                                       //
      var app = apps[appId];                                           // 85
      delete app._id;                                                  // 86
      var grains = Grains.find({                                       // 87
        lastUsed: timeConstraint,                                      // 88
        appId: appId                                                   // 89
      }, {                                                             //
        fields: { _id: 1 }                                             // 91
      }).fetch();                                                      //
      var grainIds = _.pluck(grains, "_id");                           // 93
                                                                       //
      var counts = ApiTokens.aggregate([{ $match: {                    // 95
          "owner.user.lastUsed": timeConstraint,                       // 97
          grainId: { $in: grainIds }                                   // 98
        }                                                              //
      }, { $group: { _id: "$owner.user.identityId" } }, { $group: {    //
          _id: "count",                                                // 103
          count: { $sum: 1 }                                           // 104
        }                                                              //
      }]);                                                             //
                                                                       //
      if (counts.length > 0) {                                         // 109
        if (counts.length !== 1) {                                     // 110
          console.error("error: sharedUsers aggregation returned multiple rows");
        }                                                              //
                                                                       //
        app.sharedUsers = counts[0].count;                             // 114
      }                                                                //
    }                                                                  //
                                                                       //
    // Count per-app appdemo users and deleted grains.                 //
    DeleteStats.aggregate([{ $match: {                                 // 119
        lastActive: timeConstraint,                                    // 121
        appId: { $exists: true }                                       // 122
      }                                                                //
    }, { $group: {                                                     //
        _id: {                                                         // 126
          appId: "$appId",                                             // 127
          type: "$type"                                                // 128
        },                                                             //
        count: { $sum: 1 }                                             // 130
      }                                                                //
    }]).forEach(function (deletion) {                                  //
      var app = apps[deletion._id.appId];                              // 134
      if (!app) {                                                      // 135
        app = apps[deletion.appId] = {};                               // 136
      }                                                                //
                                                                       //
      if (deletion._id.type === "appDemoUser") {                       // 139
        app.appDemoUsers = deletion.count;                             // 140
      } else if (deletion._id.type === "grain") {                      //
        app.deleted = deletion.count;                                  // 142
      } else if (deletion._id.type === "demoGrain") {                  //
        app.demoed = deletion.count;                                   // 144
      }                                                                //
    });                                                                //
                                                                       //
    return {                                                           // 148
      activeUsers: currentlyActiveUsersCount,                          // 149
      demoUsers: deletedDemoUsersCount,                                // 150
      appDemoUsers: deletedAppDemoUsersCount,                          // 151
      activeGrains: activeGrainsCount + deletedGrainsCount,            // 152
      apps: apps                                                       // 153
    };                                                                 //
  };                                                                   //
                                                                       //
  function recordStats() {                                             // 157
    var now = new Date();                                              // 158
                                                                       //
    var planStats = _.countBy(Meteor.users.find({ expires: { $exists: false }, "payments.id": { $exists: true } }, { fields: { plan: 1 } }).fetch(), "plan");
                                                                       //
    var record = {                                                     // 166
      timestamp: now,                                                  // 167
      daily: computeStats(new Date(now.getTime() - DAY_MS)),           // 168
      weekly: computeStats(new Date(now.getTime() - 7 * DAY_MS)),      // 169
      monthly: computeStats(new Date(now.getTime() - 30 * DAY_MS)),    // 170
      forever: computeStats(new Date(0)),                              // 171
      plans: planStats                                                 // 172
    };                                                                 //
    record.computeTime = Date.now() - now;                             // 174
                                                                       //
    ActivityStats.insert(record);                                      // 176
    var age = ActivityStats.find().count();                            // 177
    if (age > 3) {                                                     // 178
      var reportSetting = Settings.findOne({ _id: "reportStats" });    // 179
      if (!reportSetting) {                                            // 180
        // Setting not set yet, send out notifications and set it to false
        globalDb.sendAdminNotification("You can help Sandstorm by sending us some anonymous " + "usage stats. Click here for more info.", "/admin/stats");
        Settings.insert({ _id: "reportStats", value: "unset" });       // 184
      } else if (reportSetting.value === true) {                       //
        // The stats page which the user agreed we can send actually displays the whole history
        // of the server, but we're only sending stats from the last day. Let's also throw in the
        // length of said history. This is still strictly less information than what the user said
        // we're allowed to send.                                      //
        record.serverAge = age;                                        // 190
                                                                       //
        HTTP.post("https://alpha-api.sandstorm.io/data", {             // 192
          data: record,                                                // 193
          headers: {                                                   // 194
            Authorization: "Bearer aT-mGyNwsgwZBbZvd5FWr0Ma79O9IehI4NiEO94y_oR",
            "Content-Type": "application/json"                         // 196
          }                                                            //
        });                                                            //
      }                                                                //
    }                                                                  //
  }                                                                    //
                                                                       //
  if (!Meteor.settings.replicaNumber) {                                // 203
    // Wait until 10:00 UTC (2:00 PST / 5:00 EST), then start recording stats every 24 hours.
    // (Only on the first replica to avoid conflicts.)                 //
    Meteor.setTimeout(function () {                                    // 206
      Meteor.setInterval(function () {                                 // 207
        recordStats();                                                 // 208
      }, DAY_MS);                                                      //
                                                                       //
      recordStats();                                                   // 211
    }, DAY_MS - (Date.now() - 10 * 60 * 60 * 1000) % DAY_MS);          //
                                                                       //
    Meteor.startup(function () {                                       // 214
      if (StatsTokens.find().count() === 0) {                          // 215
        StatsTokens.remove({});                                        // 216
        StatsTokens.insert({ _id: Random.id(22) });                    // 217
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  Meteor.methods({                                                     // 222
    regenerateStatsToken: function () {                                // 223
      if (!isAdmin()) {                                                // 224
        throw new Meteor.Error(403, "Unauthorized", "User must be admin");
      }                                                                //
                                                                       //
      StatsTokens.remove({});                                          // 228
      var token = StatsTokens.insert({ _id: Random.id(22) });          // 229
      return token._id;                                                // 230
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
// Pseudo-collection defined via publish, above.                       //
RealTimeStats = new Mongo.Collection("realTimeStats");                 // 236
                                                                       //
Router.map(function () {                                               // 238
  this.route("fetchStats", {                                           // 239
    where: "server",                                                   // 240
    path: "/fetchStats/:tokenId",                                      // 241
    action: function () {                                              // 242
      var token = StatsTokens.findOne({ _id: this.params.tokenId });   // 243
                                                                       //
      if (!token) {                                                    // 245
        this.response.writeHead(404, {                                 // 246
          "Content-Type": "text/plain"                                 // 247
        });                                                            //
        this.response.write("Token not found");                        // 249
        return this.response.end();                                    // 250
      }                                                                //
                                                                       //
      try {                                                            // 253
        var stats = ActivityStats.find().fetch();                      // 254
        var statsString = JSON.stringify(stats);                       // 255
                                                                       //
        this.response.writeHead(200, {                                 // 257
          "Content-Type": "application/json"                           // 258
        });                                                            //
        this.response.write(statsString);                              // 260
      } catch (error) {                                                //
        console.error(error.stack);                                    // 262
        this.response.writeHead(500, {                                 // 263
          "Content-Type": "text/plain"                                 // 264
        });                                                            //
        this.response.write(error.stack);                              // 266
      }                                                                //
                                                                       //
      return this.response.end();                                      // 269
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
if (Meteor.isClient) {                                                 // 274
  (function () {                                                       //
    var saveReportStats = function (newValue, template) {              // 275
      var state = Iron.controller().state;                             // 276
      var token = state.get("token");                                  // 277
      template.reportStatsSaved.set(false);                            // 278
      template.fadeCheckmark.set(false);                               // 279
      if (template.fadeTimeoutId) {                                    // 280
        Meteor.clearTimeout(template.fadeTimeoutId);                   // 281
      }                                                                //
                                                                       //
      Meteor.call("setSetting", token, "reportStats", newValue, function (err) {
        if (err) {                                                     // 285
          // TODO(someday): do something with error, for now spinner will just show forever
          return;                                                      // 287
        }                                                              //
                                                                       //
        template.reportStatsSaved.set(true);                           // 290
        template.fadeTimeoutId = Meteor.setTimeout(function () {       // 291
          template.fadeCheckmark.set(true);                            // 292
        }, 1000);                                                      //
                                                                       //
        Meteor.call("dismissAdminStatsNotifications", token);          // 295
      });                                                              //
    };                                                                 //
                                                                       //
    Template.adminStats.events({                                       // 299
      "click #regenerateStatsToken": function () {                     // 300
        Meteor.call("regenerateStatsToken");                           // 301
      },                                                               //
                                                                       //
      "change select.package-date": function (ev, template) {          // 304
        template.currentPackageDate.set(ev.currentTarget.value);       // 305
      },                                                               //
                                                                       //
      "change input.enableStatsCollection": function (ev, template) {  // 308
        saveReportStats(ev.target.checked, template);                  // 309
      },                                                               //
                                                                       //
      "click .report-stats-yesno-box>.yes": function (ev, template) {  // 312
        saveReportStats(true, template);                               // 313
      },                                                               //
                                                                       //
      "click .report-stats-yesno-box>.no": function (ev, template) {   // 316
        saveReportStats(false, template);                              // 317
      }                                                                //
    });                                                                //
    Template.adminStats.onCreated(function () {                        // 320
      var _this = this;                                                //
                                                                       //
      var state = Iron.controller().state;                             // 321
      var token = state.get("token");                                  // 322
      this.currentPackageDate = new ReactiveVar(null);                 // 323
      this.reportStatsSaved = new ReactiveVar(null);                   // 324
      this.fadeCheckmark = new ReactiveVar(false);                     // 325
      this.autorun(function () {                                       // 326
        var stat = ActivityStats.findOne({}, { sort: { timestamp: -1 } });
        if (stat) {                                                    // 328
          _this.currentPackageDate.set(stat._id);                      // 329
        }                                                              //
      });                                                              //
                                                                       //
      this.subscribe("activityStats", token);                          // 333
      this.subscribe("realTimeStats", token);                          // 334
      this.subscribe("statsTokens", token);                            // 335
      this.subscribe("allPackages", token);                            // 336
    });                                                                //
                                                                       //
    Template.adminStats.helpers({                                      // 339
      setDocumentTitle: function () {                                  // 340
        document.title = "Stats · Admin · " + globalDb.getServerTitle();
      },                                                               //
                                                                       //
      points: function () {                                            // 344
        return ActivityStats.find({}, { sort: { timestamp: -1 } }).map(function (point) {
          return _.extend({                                            // 346
            // Report date of midpoint of sample period.               //
            day: new Date(point.timestamp.getTime() - 12 * 60 * 60 * 1000).toLocaleDateString()
          }, point);                                                   //
        });                                                            //
      },                                                               //
                                                                       //
      appDates: function () {                                          // 353
        var template = Template.instance();                            // 354
        return ActivityStats.find({}, { sort: { timestamp: -1 }, fields: { timestamp: 1 } }).map(function (point) {
          return _.extend({                                            // 357
            // Report date of midpoint of sample period.               //
            day: new Date(point.timestamp.getTime() - 12 * 60 * 60 * 1000).toLocaleDateString(),
            selected: point._id === template.currentPackageDate.get()  // 360
          }, point);                                                   //
        });                                                            //
      },                                                               //
                                                                       //
      apps: function () {                                              // 365
        var template = Template.instance();                            // 366
        var stats = ActivityStats.findOne({ _id: template.currentPackageDate.get() });
        if (!stats) {                                                  // 368
          return;                                                      // 369
        }                                                              //
                                                                       //
        var apps = {};                                                 // 372
        var pivotApps = function (time) {                              // 373
          var data = stats[time];                                      // 374
          if (!data) {                                                 // 375
            return;                                                    // 376
          }                                                            //
                                                                       //
          data = data.apps;                                            // 379
          for (var appId in babelHelpers.sanitizeForInObject(data)) {  // 380
            var p = data[appId];                                       // 381
            apps[appId] = apps[appId] || {};                           // 382
            apps[appId][time] = p;                                     // 383
          }                                                            //
        };                                                             //
                                                                       //
        pivotApps("daily");                                            // 387
        pivotApps("weekly");                                           // 388
        pivotApps("monthly");                                          // 389
        pivotApps("forever");                                          // 390
        return _.chain(apps).map(function (packObj, appId) {           // 391
          packObj.appId = appId;                                       // 393
          // Find the newest version of this app.                      //
          var p = Packages.findOne({                                   // 395
            appId: appId,                                              // 396
            manifest: { $exists: true }                                // 397
          }, {                                                         //
            sort: { "manifest.appVersion": -1 }                        // 399
          });                                                          //
          if (p) {                                                     // 401
            packObj.appTitle = SandstormDb.appNameFromPackage(p);      // 402
          }                                                            //
                                                                       //
          return packObj;                                              // 405
        }).sortBy(function (app) {                                     //
          return -((app.daily || {}).owners || 0);                     // 407
        }).value();                                                    //
      },                                                               //
                                                                       //
      current: function () {                                           // 411
        return RealTimeStats.findOne("now");                           // 412
      },                                                               //
                                                                       //
      today: function () {                                             // 415
        return RealTimeStats.findOne("today");                         // 416
      },                                                               //
                                                                       //
      token: function () {                                             // 419
        return StatsTokens.findOne();                                  // 420
      },                                                               //
                                                                       //
      reportStats: function () {                                       // 423
        var setting = Settings.findOne({ _id: "reportStats" });        // 424
        return setting && setting.value === true;                      // 425
      },                                                               //
                                                                       //
      reportStatsFirstVisit: function () {                             // 428
        var setting = Settings.findOne({ _id: "reportStats" });        // 429
        return !setting || setting.value === "unset";                  // 430
      },                                                               //
                                                                       //
      reportStatsSaving: function () {                                 // 433
        return !Match.test(Template.instance().reportStatsSaved.get(), Match.OneOf(undefined, null));
      },                                                               //
                                                                       //
      reportStatsSaved: function () {                                  // 437
        return Template.instance().reportStatsSaved.get();             // 438
      },                                                               //
                                                                       //
      fadeCheckmark: function () {                                     // 441
        return Template.instance().fadeCheckmark.get();                // 442
      }                                                                //
    });                                                                //
  })();                                                                //
}                                                                      //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=stats.js.map
