(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var Random = Package.random.Random;
var SandstormDb = Package['sandstorm-db'].SandstormDb;
var SandstormBackend = Package['sandstorm-backend'].SandstormBackend;
var MongoInternals = Package.mongo.MongoInternals;
var Mongo = Package.mongo.Mongo;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var check = Package.check.check;
var Match = Package.check.Match;

(function(){

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/accounts-identity/accounts-identity-server.js                                             //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Sandstorm - Personal Cloud Sandbox                                                                 // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                              // 2
// All rights reserved.                                                                               // 3
//                                                                                                    // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                    // 5
// you may not use this file except in compliance with the License.                                   // 6
// You may obtain a copy of the License at                                                            // 7
//                                                                                                    // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                       // 9
//                                                                                                    // 10
// Unless required by applicable law or agreed to in writing, software                                // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                  // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                           // 13
// See the License for the specific language governing permissions and                                // 14
// limitations under the License.                                                                     // 15
                                                                                                      // 16
function linkIdentityToAccountInternal(db, backend, identityId, accountId) {                          // 17
  // Links the identity to the account. If the account is a demo account, makes the account durable   // 18
  // and gives the identity login access to it.                                                       // 19
                                                                                                      // 20
  check(db, SandstormDb);                                                                             // 21
  check(backend, SandstormBackend);                                                                   // 22
  check(identityId, String);                                                                          // 23
  check(accountId, String);                                                                           // 24
                                                                                                      // 25
  var accountUser = Meteor.users.findOne({_id: accountId});                                           // 26
  if (!accountUser) {                                                                                 // 27
    throw new Meteor.Error(404, "No account found with ID " + accountId);                             // 28
  }                                                                                                   // 29
                                                                                                      // 30
  if (accountUser.profile) {                                                                          // 31
    throw new Meteor.Error(400, "Cannot link an identity to another identity.");                      // 32
  }                                                                                                   // 33
                                                                                                      // 34
  if (!!_.findWhere(accountUser.loginIdentities, {id: identityId}) ||                                 // 35
      !!_.findWhere(accountUser.nonloginIdentities, {id: identityId})) {                              // 36
    throw new Meteor.Error(400, "Cannot link an identity that's alread linked to this account.");     // 37
  }                                                                                                   // 38
                                                                                                      // 39
  var identityUser = Meteor.users.findOne({_id: identityId});                                         // 40
                                                                                                      // 41
  if (!identityUser) {                                                                                // 42
    throw new Meteor.Error(404, "No identity found with ID " + identityId);                           // 43
  }                                                                                                   // 44
                                                                                                      // 45
  if (!identityUser.profile) {                                                                        // 46
    throw new Meteor.Error(400, "Cannot link an account to another account");                         // 47
  }                                                                                                   // 48
                                                                                                      // 49
  db.deleteUnusedAccount(backend, identityUser._id);                                                  // 50
  if (Meteor.users.findOne({"loginIdentities.id": identityUser._id})) {                               // 51
    throw new Meteor.Error(403,                                                                       // 52
                           "Cannot link an identity that can already log into another account");      // 53
  }                                                                                                   // 54
                                                                                                      // 55
  var modifier;                                                                                       // 56
  if (accountUser.expires) {                                                                          // 57
    if (Meteor.users.findOne({"nonloginIdentities.id": identityUser._id})) {                          // 58
      throw new Meteor.Error(403, "Cannot create an account for an identity that's " +                // 59
                                  "already linked to another account.");                              // 60
    }                                                                                                 // 61
                                                                                                      // 62
    modifier = {$push: {loginIdentities: {id: identityUser._id}},                                     // 63
                $unset: {expires: 1},                                                                 // 64
                $set: {upgradedFromDemo: Date.now()}};                                                // 65
    if (Meteor.settings.public.quotaEnabled) {                                                        // 66
      // Demo users never got the referral notification. Send it now:                                 // 67
      db.sendReferralProgramNotification(accountUser._id);                                            // 68
    }                                                                                                 // 69
                                                                                                      // 70
  } else {                                                                                            // 71
    modifier = {$push: {nonloginIdentities: {id: identityUser._id}}};                                 // 72
  }                                                                                                   // 73
                                                                                                      // 74
  // Make sure not to add the same identity twice.                                                    // 75
  Meteor.users.update({_id: accountUser._id, "nonloginIdentities.id": {$ne: identityUser._id},        // 76
                       "loginIdentities.id": {$ne: identityUser._id}},                                // 77
                      modifier);                                                                      // 78
                                                                                                      // 79
  if (accountUser.expires) {                                                                          // 80
    var demoIdentityId = SandstormDb.getUserIdentityIds(accountUser)[0];                              // 81
    Meteor.users.update({_id: demoIdentityId},                                                        // 82
                        {$unset: {expires: 1},                                                        // 83
                         $set: {upgradedFromDemo: Date.now()}});                                      // 84
                                                                                                      // 85
    // Mark the demo identity as nonlogin. It'd be nicer if the identity started out as nonlogin,     // 86
    // but to get that to work we would need to adjust the account creation and first login logic.    // 87
    Meteor.users.update({_id: accountUser._id,                                                        // 88
                         "loginIdentities.id": demoIdentityId,                                        // 89
                         "nonloginIdentities.id": {$not: {$eq: demoIdentityId}}},                     // 90
                        {$pull: {loginIdentities: {id: demoIdentityId}},                              // 91
                         $push: {nonloginIdentities: {id: demoIdentityId}}});                         // 92
                                                                                                      // 93
  }                                                                                                   // 94
}                                                                                                     // 95
                                                                                                      // 96
Meteor.methods({                                                                                      // 97
  loginWithIdentity: function (accountUserId) {                                                       // 98
    // Logs into the account with ID `accountUserId`. Throws an exception if the current user is      // 99
    // not an identity user listed in the account's `loginIdentities` field. This method is not       // 100
    // intended to be called directly; client-side code should only invoke it through                 // 101
    // `Meteor.loginWithIdentity()`, which additionally maintains the standard Meteor client-side     // 102
    // login state.                                                                                   // 103
                                                                                                      // 104
    check(accountUserId, String);                                                                     // 105
                                                                                                      // 106
    var identityUser = Meteor.user();                                                                 // 107
    if (!identityUser || !identityUser.profile) {                                                     // 108
      throw new Meteor.Error(403, "Must be already logged in as an identity.");                       // 109
    }                                                                                                 // 110
                                                                                                      // 111
    var accountUser = Meteor.users.findOne(accountUserId);                                            // 112
    if (!accountUser) {                                                                               // 113
      throw new Meteor.Error(404, "No such user found: " + accountUserId);                            // 114
    }                                                                                                 // 115
                                                                                                      // 116
    var linkedIdentity = _.findWhere(accountUser.loginIdentities, {id: identityUser._id});            // 117
                                                                                                      // 118
    if (!linkedIdentity) {                                                                            // 119
      throw new Meteor.Error(403, "Current identity is not a login identity for account "             // 120
                             + accountUserId);                                                        // 121
    }                                                                                                 // 122
                                                                                                      // 123
    return Accounts._loginMethod(this, "loginWithIdentity", [accountUserId],                          // 124
                                 "identity", function () { return { userId: accountUserId }; });      // 125
  },                                                                                                  // 126
                                                                                                      // 127
  createAccountForIdentity: function() {                                                              // 128
    // Creates a new account for the currently-logged-in identity.                                    // 129
                                                                                                      // 130
    var user = Meteor.user();                                                                         // 131
    if (!(user && user.profile)) {                                                                    // 132
      throw new Meteor.Error(403, "Must be logged in as an identity in order to create an account.");
    }                                                                                                 // 134
                                                                                                      // 135
    if (Meteor.users.findOne({$or: [{"loginIdentities.id": user._id},                                 // 136
                                    {"nonloginIdentities.id": user._id}]})) {                         // 137
      throw new Meteor.Error(403, "Cannot create an account for an identity that's already " +        // 138
                                  "linked to another account.");                                      // 139
    }                                                                                                 // 140
                                                                                                      // 141
    var newUser = {loginIdentities: [{id: user._id}],                                                 // 142
                   nonloginIdentities: []};                                                           // 143
    if (user.services.dev) {                                                                          // 144
      newUser.signupKey = "devAccounts";                                                              // 145
      if (user.services.dev.isAdmin) {                                                                // 146
        newUser.isAdmin = true;                                                                       // 147
      }                                                                                               // 148
      if (user.services.dev.hasCompletedSignup) {                                                     // 149
        newUser.hasCompletedSignup = true;                                                            // 150
      }                                                                                               // 151
    } else if (user.expires) {                                                                        // 152
      // Demo user.                                                                                   // 153
      newUser.expires = user.expires;                                                                 // 154
      if (!!user.appDemoId) {                                                                         // 155
        newUser.appDemoId = user.appDemoId;                                                           // 156
      }                                                                                               // 157
    }                                                                                                 // 158
    var options = {};                                                                                 // 159
                                                                                                      // 160
    // This will throw an error if the identity has been added as a login identity to some            // 161
    // other account while we were executing the body of this method.                                 // 162
    return Accounts.insertUserDoc(options, newUser);                                                  // 163
  },                                                                                                  // 164
                                                                                                      // 165
  linkIdentityToAccount: function(token) {                                                            // 166
    // Links the identity of the current user to the account that has `token` as a resume token.      // 167
    // If the account is a demo account, makes the account durable and gives the identity login       // 168
    // access to it.                                                                                  // 169
                                                                                                      // 170
    check(token, String);                                                                             // 171
                                                                                                      // 172
    if (!this.userId) {                                                                               // 173
      throw new Meteor.Error(403, "Cannot link to account if not logged in.");                        // 174
    }                                                                                                 // 175
    var hashed = Accounts._hashLoginToken(token);                                                     // 176
    var accountUser = Meteor.users.findOne({"services.resume.loginTokens.hashedToken": hashed});      // 177
                                                                                                      // 178
    linkIdentityToAccountInternal(this.connection.sandstormDb, this.connection.sandstormBackend,      // 179
                                  this.userId, accountUser._id);                                      // 180
  },                                                                                                  // 181
                                                                                                      // 182
  unlinkIdentity: function (accountUserId, identityId) {                                              // 183
    // Unlinks the identity with ID `identityId` from the account with ID `accountUserId`.            // 184
                                                                                                      // 185
    check(identityId, String);                                                                        // 186
    check(accountUserId, String);                                                                     // 187
                                                                                                      // 188
    if (!this.userId) {                                                                               // 189
      throw new Meteor.Error(403, "Not logged in.");                                                  // 190
    }                                                                                                 // 191
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                      // 192
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                // 193
    }                                                                                                 // 194
                                                                                                      // 195
    var identityUser = Meteor.users.findOne({_id: identityId});                                       // 196
    Meteor.users.update({_id: accountUserId},                                                         // 197
                        {$pull: {nonloginIdentities: {id: identityId},                                // 198
                                 loginIdentities: {id: identityId}}});                                // 199
  },                                                                                                  // 200
                                                                                                      // 201
  setIdentityAllowsLogin: function(identityId, allowLogin) {                                          // 202
    // Sets whether the current account allows the identity with ID `identityId` to log in.           // 203
                                                                                                      // 204
    check(identityId, String);                                                                        // 205
    check(allowLogin, Boolean);                                                                       // 206
    if (!this.userId) {                                                                               // 207
      throw new Meteor.Error(403, "Not logged in.");                                                  // 208
    }                                                                                                 // 209
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                      // 210
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                // 211
    }                                                                                                 // 212
                                                                                                      // 213
    if (allowLogin) {                                                                                 // 214
      Meteor.users.update({_id: this.userId,                                                          // 215
                           "nonloginIdentities.id": identityId,                                       // 216
                           "loginIdentities.id": {$not: {$eq: identityId}}},                          // 217
                          {$pull: {nonloginIdentities: {id: identityId}},                             // 218
                           $push: {loginIdentities: {id: identityId}}});                              // 219
    } else {                                                                                          // 220
      Meteor.users.update({_id: this.userId,                                                          // 221
                           "loginIdentities.id": identityId,                                          // 222
                           "nonloginIdentities.id": {$not: {$eq: identityId}}},                       // 223
                          {$pull: {loginIdentities: {id: identityId}},                                // 224
                           $push: {nonloginIdentities: {id: identityId}}});                           // 225
    }                                                                                                 // 226
  },                                                                                                  // 227
                                                                                                      // 228
  logoutIdentitiesOfCurrentAccount: function() {                                                      // 229
    // Logs out all identities that are allowed to log in to the current account.                     // 230
    var user = Meteor.user();                                                                         // 231
    if (user && user.loginIdentities) {                                                               // 232
      user.loginIdentities.forEach(function(identity) {                                               // 233
        Meteor.users.update({_id: identity.id}, {$set: {"services.resume.loginTokens": []}});         // 234
      });                                                                                             // 235
    }                                                                                                 // 236
  }                                                                                                   // 237
});                                                                                                   // 238
                                                                                                      // 239
Accounts.linkIdentityToAccount = function (db, backend, identityId, accountId) {                      // 240
  // Links the identity to the account. If the account is a demo account, makes it durable and        // 241
  // gives the identity login access to it.                                                           // 242
  check(db, SandstormDb);                                                                             // 243
  check(backend, SandstormBackend);                                                                   // 244
  check(identityId, String);                                                                          // 245
  check(accountId, String);                                                                           // 246
  linkIdentityToAccountInternal(db, backend, identityId, accountId);                                  // 247
}                                                                                                     // 248
                                                                                                      // 249
Meteor.publish("accountsOfIdentity", function (identityId) {                                          // 250
  check(identityId, String);                                                                          // 251
  if (!SandstormDb.ensureSubscriberHasIdentity(this, identityId)) return;                             // 252
                                                                                                      // 253
  // We maintain a map from identity IDs to live query handles that track profile changes.            // 254
  var loginIdentities = {};                                                                           // 255
                                                                                                      // 256
  var self = this;                                                                                    // 257
  function addIdentitiesOfAccount(account) {                                                          // 258
    account.loginIdentities.forEach(function(identity) {                                              // 259
      if (!(identity.id in loginIdentities)) {                                                        // 260
        var user = Meteor.users.findOne({_id: identity.id});                                          // 261
        if (user) {                                                                                   // 262
          SandstormDb.fillInProfileDefaults(user);                                                    // 263
          SandstormDb.fillInIntrinsicName(user);                                                      // 264
          var filteredUser = _.pick(user, "_id", "profile");                                          // 265
          filteredUser.loginAccountId = account._id;                                                  // 266
          filteredUser.sourceIdentityId = identityId;                                                 // 267
          self.added("loginIdentitiesOfLinkedAccounts", user._id, filteredUser);                      // 268
        }                                                                                             // 269
        loginIdentities[identity.id] =                                                                // 270
          Meteor.users.find({_id: identity.id}, {fields: {profile: 1}}).observeChanges({              // 271
            changed: function (id, fields) {                                                          // 272
              self.changed("loginIdentitiesOfLinkedAccounts", id, fields);                            // 273
            }                                                                                         // 274
          });                                                                                         // 275
      }                                                                                               // 276
    });                                                                                               // 277
  }                                                                                                   // 278
  var cursor = Meteor.users.find({$or: [{"loginIdentities.id": identityId},                           // 279
                                        {"nonloginIdentities.id": identityId}]});                     // 280
                                                                                                      // 281
  var handle = cursor.observe({                                                                       // 282
    added: function (account) {                                                                       // 283
      addIdentitiesOfAccount(account);                                                                // 284
    },                                                                                                // 285
    changed: function (newAccount, oldAccount) {                                                      // 286
      addIdentitiesOfAccount(newAccount);                                                             // 287
    },                                                                                                // 288
    removed: function (account) {                                                                     // 289
      account.loginIdentities.forEach(function(identity) {                                            // 290
        if (identity.id in loginIdentities) {                                                         // 291
          self.removed("loginIdentitiesOfLinkedAccounts", identity.id);                               // 292
          loginIdentities[identity.id].stop();                                                        // 293
          delete loginIdentities[identity.id];                                                        // 294
        }                                                                                             // 295
      });                                                                                             // 296
    },                                                                                                // 297
  });                                                                                                 // 298
  this.ready();                                                                                       // 299
                                                                                                      // 300
  this.onStop(function() {                                                                            // 301
    handle.stop();                                                                                    // 302
    Object.keys(loginIdentities).forEach(function(identityId) {                                       // 303
      loginIdentities[identityId].stop();                                                             // 304
      delete loginIdentities[identityId];                                                             // 305
    });                                                                                               // 306
  });                                                                                                 // 307
});                                                                                                   // 308
                                                                                                      // 309
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['accounts-identity'] = {};

})();

//# sourceMappingURL=accounts-identity.js.map
